﻿Camera = function(renderObject){
	this.renderObject = renderObject;
	this.position = new Vec2 (Game.width/2, Game.height/2);
	this.zoom = 1;
	this.windowSize = new Vec2(Game.width/2, Game.heightShift/2);
	this.changeRender();
	Game.onresize.push(function () {
	    this.windowSize = new Vec2(Game.width / 2, Game.heightShift / 2);
	    this.move(new Vec2(0, 0));
	    this.changeRender();
	}.bind(this));
   this.onScreen = this.getMovedPosition(new Vec2(1,1));
 };

Camera.prototype.changeRender = function(){
	this.renderObject.pivot = this.getShift();
	this.renderObject.scale.x = this.renderObject.scale.y = this.getScale();
};

Camera.prototype.getScale = function(){
	return this.zoom * Game.scale;
};

Camera.prototype.getShift = function(){
	return this.position.clone().sub(this.windowSize.clone().multiply(1/this.zoom)).multiply(1);
};

Camera.prototype.move = function(direction){
    this.position.add(direction.multiplyed(1/this.zoom));
    this.checkViewport();
	this.changeRender();
	if (!Game.multiplayerLvl && Game.lvl.gameLogic.state == 'day')
		Game.lvl.objEffect.forEach(function(eff){
			if (eff.updatePosition)
				eff.updatePosition();
		});
    this.onScreen.set(this.getMovedPosition(new Vec2(1,1)));
};


Camera.prototype.scale = function(scaleFactor, direction) {
    if (scaleFactor > 0 && Game.lvl.paused)
        return;

    if (isNaN(scaleFactor))
        return;

	if (Math.abs(scaleFactor) < 0.05)
		scaleFactor = 0.0;

	if ((scaleFactor == Infinity)||(scaleFactor==-Infinity))
		scaleFactor = 0.0;
  	this.zoom += scaleFactor;
	if (this.zoom < 1) 
		this.zoom = 1;
	else if (this.zoom > 1.3)
		this.zoom = 1.3;

    if (Game.isLowMemory || Game.pc)
        this.zoom = 1;

	if ((direction)&&(scaleFactor!=0.0))
  		 this.move(this.getMovedPosition(direction).sub(this.position.clone()).multiplyed(scaleFactor));
   
	this.checkViewport();
	this.changeRender();
    this.onScreen.set(this.getMovedPosition(new Vec2(1,1)));
};

Camera.prototype.checkViewport = function(){
    var viewPortLeft = this.position.clone().sub(this.windowSize.multiplyed(1/this.zoom));
    var viewPortRight = this.position.clone().add(this.windowSize.multiplyed(1/this.zoom));
    if(viewPortLeft.x< 0){
	    this.position.x = this.windowSize.multiplyed(1/this.zoom).x;
    }
    if (viewPortLeft.y < 0){
	    this.position.y = this.windowSize.multiplyed(1/this.zoom).y;
    }
    if(viewPortRight.x > this.windowSize.x*2){
	    this.position.x = this.windowSize.x*2 - this.windowSize.multiplyed(1/this.zoom).x;
    }
    if (viewPortRight.y > Game.height){
	    this.position.y = Game.height - this.windowSize.multiplyed(1/this.zoom).y;
    }
};

Camera.prototype.getMovedPosition = function(position){
    return position.clone().multiply(1/this.zoom).add(this.position.clone().sub(this.windowSize.multiplyed(1/this.zoom)));
};

Camera.prototype.getUnMovedPosition = function(position){
    return position.clone().sub(this.position.clone().sub(this.windowSize.multiplyed(1/this.zoom))).multiply(this.zoom);
};

Camera.prototype.getWaveIconPosition = function(position, bonusLineButtons) {
    var pause = Game.lvl.camera.getMovedPosition(new Vec2(1024, 0));
	if (position.x < this.position.x - (this.windowSize.x - 80)/this.zoom)
		position.x = this.position.x - (this.windowSize.x - 80)/this.zoom;

	if (position.x > this.position.x + (this.windowSize.x - 80)/this.zoom)
		position.x = this.position.x + (this.windowSize.x - 80)/this.zoom;

	if (position.y < this.position.y - (this.windowSize.y - 80)/this.zoom)
		position.y = this.position.y - (this.windowSize.y - 80)/this.zoom;

	if (position.y > this.position.y + (this.windowSize.y - 80)/this.zoom)
		position.y = this.position.y + (this.windowSize.y - 80)/this.zoom;


	if (position.x- (this.position.x - this.windowSize.x/this.zoom)  < 650/this.zoom && position.y - (this.position.y - this.windowSize.y/this.zoom) < 220/this.zoom) {
		//if (position.y - (this.position.y - this.windowSize.y/this.zoom) < 400/this.zoom)
		//	position.x = (this.position.x - this.windowSize.x/this.zoom) + 450/this.zoom;
        //
		//else if (position.x - (this.position.x - this.windowSize.x/this.zoom)  < 450)
        position.y = (this.position.y - this.windowSize.y/this.zoom) + 220/this.zoom;
	} else if (position.x - (this.position.x - this.windowSize.x/this.zoom) > 1800/this.zoom && position.y - (this.position.y - this.windowSize.y/this.zoom) < 260/this.zoom) {

		if (position.y - (this.position.y - this.windowSize.y/this.zoom) < 200/this.zoom)
			position.x = (this.position.x - this.windowSize.x/this.zoom) + 1800/this.zoom;

		else if (position.x - (this.position.x - this.windowSize.x/this.zoom) > 1600/this.zoom)
			position.y = (this.position.y - this.windowSize.y/this.zoom) + 260/this.zoom;

	} else if (position.x - (this.position.x - this.windowSize.x/this.zoom) > (Game.width - 280)/this.zoom && position.y - (this.position.y - this.windowSize.y/this.zoom) > (Game.heightShift - bonusLineButtons*150 + 80 - 200)/this.zoom ) {
        //bonus
        position.x = (this.position.x + this.windowSize.x/this.zoom) - (280)/this.zoom ;

        //if (position.x > (this.position.x - this.windowSize.x/this.zoom) + (Game.width - 220)/this.zoom - 80)
        if (position.y - (this.position.y - this.windowSize.y/this.zoom) > (Game.heightShift - 200)/this.zoom)
            position.y = (this.position.y + this.windowSize.y/this.zoom) - (200)/this.zoom;

	} else if (position.x - (this.position.x - this.windowSize.x/this.zoom) < (400)/this.zoom  && position.y - (this.position.y - this.windowSize.y/this.zoom) > (Game.heightShift - 400)/this.zoom) {

        position.x = (this.position.x - this.windowSize.x/this.zoom) + (230)/this.zoom;
        position.y = (this.position.y + this.windowSize.y/this.zoom) - (210)/this.zoom;

        //if (position.x > (this.position.x - this.windowSize.x/this.zoom) + (Game.width - 220)/this.zoom - 80)
        //if (position.y - (this.position.y - this.windowSize.y/this.zoom) > (Game.heightShift - 200)/this.zoom)
        //    position.y = (this.position.y + this.windowSize.y/this.zoom) - (200)/this.zoom;

    } else if (Math.abs(position.x - pause.x) < 180/this.zoom &&  position.y - (this.position.y - this.windowSize.y/this.zoom) < 260/this.zoom) {
        position.x = pause.x - 180/this.zoom;
    }
    return position;
};
