﻿function Enemy(params, x, y, strategy, spawnTag) {

    this.bufs = [];
    this.position = new Vec2(x, y);
    this.animations = params.animations;
    this.animation = Game.lvl.pool.getAnimation(params.animations.run);
    Game.lvl.gameContainer.addChild(this.animation);
    this.animation.position = this.position;
    this.base = params;
    this.bounty = this.base.bounty;
    this.animation.setDirection(1);
    this.timeOut = params.timeOut/3;
    this.scanTimeOut = 0;
    this.invulnerable = 0;
    this.frezeTimeout = 0;
    this.moveRange = 0;
    this.attackRange = params.attackRange;
    this.hitPosition = params.hitPosition || new Vec2(0,0);
    this.hitRange = params.hitRange || 0;
    this.speed = params.speed;
    this.hitPoints = params.hitPoints*Game.lvl.gameLogic.hitPointsCounter*Game.globallogic.getHitpointsModificator();
    this.maxHitPoints = params.hitPoints*Game.lvl.gameLogic.hitPointsCounter*Game.globallogic.getHitpointsModificator();
    this.armor = params.armor || 0;
    this.magicArmor =  params.magicArmor || 0;
    this.spawnTag = spawnTag || null;

    if (this.spawnTag === null)
        throw new Error('Unknown spawn tag');

    this.directionDirect = false;
    this.currectTarget = null;
    this.path = [];
    this.penetration = params.penetration === 1;
    this.tileRandomizer = new Vec2(32-myRandom(16,48), 32-myRandom(16,48));
    this.damage = params.damage;
    this.fightAnim = 0;
    this.tile = null;
    this.strategyType =  'TownHall';

    if (params.flying) {
        this.flying = true;
        this.strategy = Enemy.prototype.flyingStrategy;
        this.path = Game.lvl.objMap.getPathForFlying(this.position, Game.lvl.townHall.position.clone().add(Game.lvl.townHall.hitPosition), this);
        this.directionDirect = true;
    } else {
        this.flying = false;
        this.target = null;
        this.searchPath();
    }

    this.init();
    this.type = 'enemy';
    this.parentStrategy = params.strategy;

    if (params.init)
        params.init.call(this);

    if (params.soundDeath)
        this.soundDeath = Game.lvl.audioPool.get(params.soundDeathGroup, params.soundDeath, params.soundDeathVolume || 1, params.soundDeathLimit);

    if (params.soundCC)
        this.soundCC = Game.lvl.audioPool.get(params.soundCCGroup, params.soundCC, params.soundCCVolume || 1, params.soundCCLimit);

    if (params.soundShoot)
        this.soundShoot = Game.lvl.audioPool.get(params.soundShootGroup, params.soundShoot, params.soundShootVolume || 1, params.soundShootLimit);
}

Enemy.prototype.switchFight  = function () {
    if (this.animation.name == this.animations.fight[this.fightAnim] ) {
        var newFightAnim = myPofigRandom(0, this.animations['fight'].length);
        if (this.fightAnim != newFightAnim) {
            this.fightAnim = newFightAnim;
            this.nextAnimation = this.animations.fight[this.fightAnim];
        }
    }
};

Enemy.prototype.update  = function () {
    var key;

    for (key in this.bufs) {
        if (this.bufs.hasOwnProperty(key) && this.bufs[key] != null)
            this.bufs[key].update();
    }

    if (this.frezeTimeout > 0) {
        this.frezeTimeout -= Game.cycleTickTime*Game.speed;
        if (this.frezeTimeout <= 0) {
            this.animation.play();
        } else
            return;
    }

    if (this.invulnerable > 0)
        this.invulnerable -= Game.cycleTickTime*Game.speed;

    if (this.hitPoints > 0) {
        if (this.parentStrategy)
            this.parentStrategy.call(this);
        else
            this.strategy();

        if (this.nextAnimation) {
            this.animation = Game.lvl.pool.switchAnimation(this.nextAnimation, this.animation);
            this.nextAnimation = null;
        }
    }
};

Enemy.prototype.init  = function () {
    if (this.animations.fight.length > 1) {
        this.animation.onComplete = this.switchFight.bind(this);
    }
    this.tile = Game.lvl.objMap.getTile(this.position);
    if (this.tile)
        this.tile.eObjects.push(this);
};

Enemy.prototype.move = function() {
    var direction, len, target, key;
    var gameSpeed = Game.speed;
    while (--gameSpeed >= 0) {
        while ((this.path.length > 0) && ((direction = this.path[this.path.length - 1].position.clone().sub(this.position).add(this.tileRandomizer)).getLength() <= this.speed * 2)) {
            this.path.pop();
            this.animation.direction = direction.getDirection();
        }

        if (this.path.length > 0) {
            this.position.add(direction.normalize().multiply(this.speed));
            this.animation.setDirection(direction.getDirection());
            this.animation = Game.lvl.pool.switchAnimation(this.animations['run'], this.animation);
            if (!this.checkMove()) {
                this.position.sub(direction);
                break;
            }
        } else {
            if (Game.lvl.townHall.hitPoints > 0) {
                Game.lvl.townHall.applyDamage(this.damage, this);
                break;
            }
        }
    }
};

Enemy.prototype.fantomMove = function(iter){
    var direction, len, target, key;

    if (this.target || this.hitPoints <= 0)
        return this.position;

    var position = this.position.clone();
    var path = this.path.slice(0);
    for (var i = 0; i < iter; i++) {
        if (path.length > 0 &&((direction = path[path.length-1].position.clone().sub(position).add(this.tileRandomizer)).getLength() <= this.speed*2)){
            path.pop();
        }
        if (path.length > 0) {
            position.add(direction.normalize().multiply(this.speed));
        }
    }
    return position;
};

Enemy.prototype.del = function () {
    if (this.tile && this.tile.eObjects.indexOf(this) != -1)
        this.tile.eObjects.splice(this.tile.eObjects.indexOf(this), 1);

    this.hitPoints = -1;
    this.alreadyDead = true;
    setTimeout(function () {
        if (Game.lvl.objEnemy.indexOf(this)!=-1)
            Game.lvl.objEnemy.splice(Game.lvl.objEnemy.indexOf(this), 1);

        if (Game.lvl.objEffect.indexOf(this)!=-1)
            Game.lvl.objEffect.splice(Game.lvl.objEffect.indexOf(this), 1);

        Game.lvl.pool.pullAnimation(this.animation);

        for (var key in this.bufs) {
            if (this.bufs.hasOwnProperty(key) && this.bufs[key] != null)
                this.bufs[key].deleteBuf();
        }
    }.bind(this), 1);
};

Enemy.prototype.checkMove = function (){
    var newTile = Game.lvl.objMap.getTile(this.position);
    if (newTile!= this.tile) {
        if (newTile.objects.length > 0 && newTile.objects[0]==Game.lvl.townHall) {
            this.animation = Game.lvl.pool.switchAnimation(this.animations['fight'][this.fightAnim], this.animation);
            this.timeOut = this.base.timeOut;
            Game.lvl.townHall.applyDamage(this.damage, this);
            return false;
        }
        if (this.tile)
            this.tile.eObjects.splice(this.tile.eObjects.indexOf(this), 1);
        this.tile = newTile;
        this.tile.eObjects.push(this);
    }
    return true;
};

Enemy.prototype.applyDamage = function (damage, source) {

    var armor = source && source.damageType == 'magic' ? this.magicArmor : this.armor;
    var penetration = source ? source.penetration || 0 : 1;
    if (source && source.criticalChance && checkCritical(source.criticalChance)){
        //var crit = true;
        var crit = new CritBlood(this.position.clone().add(this.hitPosition));
        Game.lvl.objEffect.push(crit);
        damage *= 2;
        penetration = 1;
        Game.events.emit('criticalStrike');
    }

    if (this.invulnerable <= 0)
        this.hitPoints -= damage*(1 - armor*(1-penetration));

    if (this.hitPoints<=0) {
        if(!this.alreadyDead) {
            this.soundDeath && this.soundDeath.play();
            Game.events.emit('enemyKilled');
            if (source === Game.lvl.gameLogic.hero) {
                Game.lvl.gameLogic.lazarus = false;
                Game.events.emit('enemyKilledFromHero');
                Game.lvl.gameLogic.hero.addExperience(this.base.bounty || 0);
            } else if (source && source.base.subtype)
                Game.events.emit(source.base.subtype + 'Kill');

         if (this.base.subtype)
            Game.events.emit(this.base.subtype + 'Killer');


        if (this.flying && source && source.base.subtype2 == 'horde')
                Game.events.emit('hordeFlyingKill');

            if (crit)
                Game.events.emit('enemyKilledByCrit');

            this.onDeath();
            this.alreadyDead = true;
        }
    } else {
        if (!this.hitPointsEffect)
            this.hitPointsEffect = new hitPointsEffect(this);
    }
};

Enemy.prototype.onDeath = function(){
    if (this.base.onDeath)
        this.base.onDeath.call(this);

    Game.lvl.gameLogic.addMoney(this.bounty);

    if (this.base.bounty > 100)
        Game.lvl.objEffect.push(new moneyEffect(this.base.bounty, Game.lvl.camera.getUnMovedPosition(this.position.clone().add(new Vec2(0, - 32)))));

    if (Game.lvl.objEnemy.indexOf(this)!=-1)
        Game.lvl.objEnemy.splice(Game.lvl.objEnemy.indexOf(this), 1);

    this.animation = Game.lvl.pool.switchAnimation(this.animations['die'], this.animation);
    this.animation.randomizeTime = 0;
    this.animation.loop = false;
    this.animation.onComplete = function() {
        this.del();
        Game.lvl.objMap.drawOnBackground(Game.sprites['skeleton'+[myPofigRandom(0,3)]], this.position.x - 32, this.position.y-32, 1);
    }.bind(this);
};

Enemy.prototype.scanEnemyAround = function (){
    var direction;

    if (this.target) {
        direction = this.target.position.clone().sub(this.position);
        if (this.target.hitPoints <= 0|| ((direction.getLength() > this.attackRange*2 || this.target.target != this)&&(this.target instanceof Friendly)))
            this.target = null;
        else if (this.target.hitPoints > 0) {
            return;
        }
    }

    if (this.scanTimeOut > 0)
        return;

    this.scanTimeOut = this.base.timeOut;

    Game.lvl.objFriendly.forEach(function (enemy) {
        if (enemy.target == this && enemy.position.clone().sub(this.position).getLength() < this.attackRange*2 && enemy.hitPoints > 0) {
            this.target = enemy;
        }
    }.bind(this));
};

Enemy.prototype.scanEnemyAroundArcher = function (){
    var direction;

    if (this.target) {
        direction = this.target.position.clone().sub(this.position);
        if (this.target.hitPoints <= 0|| direction.getLength() > this.attackRange || this.target.target != this)
            this.target = null;
        else
            return;
    }

    if (this.scanTimeOut > 0)
        return;

    var target = null;
    this.scanTimeOut = this.timeOut/4;
    Game.lvl.objFriendly.forEach(function(enemy) {
        if (!this.target)
            if (enemy.position.clone().sub(this.position).getLength() < this.attackRange && enemy.hitPoints > 0) {
                if (enemy.target === this)
                    this.target = enemy;
                else
                    target = enemy;
            }
    }.bind(this));

    if (!this.target)
        this.target = target;
};


Enemy.prototype.strategy = function() {
    this.scanTimeOut -= Game.cycleTickTime*Game.speed;
    this.timeOut -= Game.cycleTickTime*Game.speed;

    if (this.timeOut > 0)
        return;

    this.scanEnemyAround();

    if (this.target) {
        this.animation = Game.lvl.pool.switchAnimation(this.animations['fight'][this.fightAnim], this.animation);
        this.timeOut = this.base.timeOut;
        this.target.applyDamage(this.damage, this);
        this.animation.setDirection(this.target.position.clone().sub(this.position).getDirection());
        this.soundCC && this.soundCC.play();
    } else {
        this.animation = Game.lvl.pool.switchAnimation(this.animations['run'], this.animation);
        if (this.path.length > 0)
            this.move();
    }
};

Enemy.prototype.flyingStrategy = function () {
    this.timeOut -= Game.cycleTickTime*Game.speed;
    if (this.timeOut > 0)
        return;
    if (this.target && this.target.hitPoints > 0) {
        this.animation = Game.lvl.pool.switchAnimation(this.animations['fight'][this.fightAnim], this.animation);
        this.timeOut = this.base.timeOut;
        this.target.applyDamage(this.damage, this);
    } else {
        if (this.path.length > 0)
            this.move();
        this.animation = Game.lvl.pool.switchAnimation(this.animations['run'], this.animation);
    }
};

Enemy.prototype.searchPath = function () {
    this.currectTarget = Game.lvl.townHall;
    this.path = Game.lvl.objMap.getPathForMonster(this.position, Game.lvl.townHall.position.clone().add(Game.lvl.townHall.hitPosition), this);
};