﻿function Friendly(params, x, y, parent) {
	this.base = params;
    this.parent = parent || null;
    if (Game.multiplayerController)
        this._id = Game.lvl.getId();
	this.bufs = [];
	this.moveRange = params.moveRange;
	this.criticalChance = params.criticalChance || 0;
	this.attackRange = params.attackRange;
	this.animations = params.animations;
	this.animation = Game.lvl.pool.getAnimation(params.animations.idle);
	Game.lvl.gameContainer.addChild(this.animation);
	this.armor = params.armor;
	this.penetration = params.penetration;
	this.speed = params.speed;
	this.basePosition = new Vec2(x, y);
	this.position = this.basePosition.clone();
    this.basePositionMoved = false;
    this.animation.position = this.position;
	this.hitPosition = params.hitPosition || new Vec2(0,0);
    this.tileSdvig = new Vec2(0, 0);
	this.hitRange = params.hitRange || 0;
	this.hitPoints = params.hitPoints;
	this.maxHitPoints = params.hitPoints;
    this.damageType = params.damageType || 'normal';
	this.damage = params.damage;
	this.fightAnim = 0;
    this.deadTimer = 0;
	this.path = [];
    this.invulnerable = 0;
	if (params.height)
		this.height = params.height;
	else
		this.height = 0;
	this.tile = null;
	this.parentStrategy = params.strategy;
	this.animation.setDirection (1);
	this.tiles = [];
	this.timeOut = 0;
	this.scanTimeOut = 0;
	this.type = 'friendly';
	this.cluster = null;
	this.init();

	if (params.init)
		params.init.call(this);

    if (params.onDeath)
	    this.onDeath = params.onDeath;

    if (params.unitName == 'hero') {
        this.heroInit();
        this.isHero = true;
    }

    if (params.soundShoot)
        this.soundShoot = Game.lvl.audioPool.get(params.soundShootGroup, params.soundShoot, params.soundShootVolume || 1, params.soundShootLimit);

    if (params.soundCC)
        this.soundCC = Game.lvl.audioPool.get(params.soundCCGroup, params.soundCC, params.soundCCVolume || 1, params.soundCCLimit);

    if (params.soundDeath)
        this.soundDeath = Game.lvl.audioPool.get(params.soundDeathGroup, params.soundDeath, params.soundDeathVolume || 1, params.soundDeathLimit);

    if (params.soundOrder)
        this.soundOrder = Game.lvl.audioPool.get(params.soundOrderGroup, params.soundOrder, params.soundOrderVolume || 1, params.soundOrderLimit);
}

Friendly.prototype.heroInit = function () {
    var hero = Game.globallogic.selectedHero;
    if (hero) {
        this.deathCounter = 0;
        this.damage = this.base.damage + this.base.damage*(hero.lvl/10);
        this.maxHitPoints = this.base.hitPoints + this.base.hitPoints*(hero.lvl/10);
        this.hitPoints = this.maxHitPoints;
        this.applyDamage = this.applyDamageHero;
        this.isHero = true;
        this.abilityTrain = Game.menu.selectedLvl === "0";
        if (this.abilityTrain)
            this.moveRange += 64;
    }
};

Friendly.prototype.addExperience = function (exp) {
    if (Game.globallogic.selectedHero.addExperience(exp)) {
        this.heroInit();
        Game.lvl.audioPool.get('LvL_UP', 'LvL_UP', 0.3, 3).play();
        Game.lvl.objEffect.push(new EffectSmall(this.position, Game.lvl.effectContainer, Game.animations['LVLUP'],Game.animations['LVLUP'].playTime, false));
    }
};

Friendly.prototype.onDeath = function () {
    this.animation = Game.lvl.pool.switchAnimation(this.animations['die'], this.animation);
    this.animation.loop = false;
    this.animation.randomizeTime = 0;
    if (this.moveRange > 0)
        this.cluster.release();
    this.deadTimer = 35*20;
    if (this.isHero) {
        this.deadTimer-= (Game.globallogic.getTownHallLvl()-1)*20*3;
    }
    this.animation.onComplete = function(){}.bind(this);
};

Friendly.prototype.onDay = function(){
};

Friendly.prototype.onNight = function(){
};

Friendly.prototype.update  = function () {
	var key;

    for (key in this.bufs){
        if (this.bufs.hasOwnProperty(key) && this.bufs[key] != null)
            this.bufs[key].update();
    }

    if (this.hitPoints > 0 && this.deadTimer <= 0) {
        if (this.hitPoints < this.maxHitPoints)
            this.hitPoints += this.maxHitPoints*Game.speed/(20*60);

		if (this.abilityPlay > 0) {
			this.abilityPlay -= Game.cycleTickTime*Game.speed;
			return;
		}
			if (this.parentStrategy)
				this.parentStrategy.call(this);
			else
				this.strategy();

		if (this.nextAnimation) {
			this.animation = Game.lvl.pool.switchAnimation(this.nextAnimation, this.animation);
			this.nextAnimation = null;
		}
	} else if ((this.deadTimer-=Game.speed) <= 0 && (!this.house || this.house.disabled <=0)) {
        this.alreadyDead = false;
        this.hitPoints = this.maxHitPoints;
        this.movePosition(this.basePosition, true);
        this.timeOut = 0;
    }

	if (this.toDelete) {
        this.fullDelete();
	}
};

Friendly.prototype.fullDelete = function() {
    if (Game.lvl.objFriendly.indexOf(this) != -1)
        Game.lvl.objFriendly.splice(Game.lvl.objFriendly.indexOf(this), 1);

    Game.lvl.pool.pullAnimation(this.animation);

    if (this.hitPointsEffect)
        this.hitPointsEffect.del();

    this.hitPoints = -1;

    for (var key in this.bufs){
        if (this.bufs.hasOwnProperty(key) && this.bufs[key] != null)
            this.bufs[key].deleteBuf();
    }

    if (this.moveRange > 0 && this.cluster.tiles.length > 0)
        this.cluster.release();

    if (this.tile.wObjects.indexOf(this) != -1)
        this.tile.wObjects.splice(this.tile.wObjects.indexOf(this), 1);
};

Friendly.prototype.init  = function () {
	if (this.animations.fight.length > 1){
		var that = this;
		this.animation.onComplete = function() {
			if (that.animation.name === that.animations.fight[that.fightAnim]) {
				var newFightAnim = parseInt(myPofigRandom(0, that.animations['fight'].length));
				if (that.fightAnim != newFightAnim){
					that.fightAnim = newFightAnim;
					that.nextAnimation = that.animations.fight[that.fightAnim];
				}
			}
		};
	}

	this.tile = Game.lvl.objMap.getTile(this.basePosition);
	this.tile.wObjects.push(this);
	if (this.moveRange > 0)
		this.initCluster();
};

Friendly.prototype.initCluster = function(){
	if (this.cluster)
		this.cluster = this.cluster.init(Game.lvl.objMap.getTile(this.basePosition),  this.moveRange + this.attackRange);
	else
		this.cluster = new Cluster(Game.lvl.objMap.getTile(this.basePosition),  this.moveRange + this.attackRange);
};

Friendly.prototype.movePosition = function(position, afterDeath) {
    var tile =  Game.lvl.objMap.getTile(position);
    if (this.hitPoints > 0) {
        !afterDeath && this.soundOrder && this.soundOrder.play();
		this.target = null;
		this.path = [];
        if (!tile.isGlobalyWalkable)
            this.basePosition = Game.lvl.objMap.getClosesEmptyTile(position).position.clone();
        else
            this.basePosition = tile.position;
		this.initCluster();
		this.timeOut = 0;
        this.basePositionMoved = true;
		this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
	} else {
        this.basePosition = Game.lvl.objMap.getClosesEmptyTile(position).position.clone();
    }
};

Friendly.prototype.move = function(){
	var direction;

    var gameSpeed = Game.speed;
    while (--gameSpeed >= 0) {
        while ((this.path.length > 0) && ((direction = this.path[this.path.length - 1].position.clone().sub(this.position).add(this.tileSdvig)).getLength() < this.speed)) {
            this.path.pop();
            this.animation.setDirection(direction.getDirection());
        }
        if (this.path.length > 0) {
            this.position.add(direction.normalize().multiply(this.speed));
            this.animation.setDirection(direction.getDirection());
            this.animation = Game.lvl.pool.switchAnimation(this.animations['run'], this.animation);
            this.checkMove();
        } else {
            var lng = this.basePosition.clone().add(this.tileSdvig).sub(this.position);
            if (this.tile != Game.lvl.objMap.getTile(this.position) && this.position.clone().sub(this.tileSdvig).sub(this.basePosition).getLength() > this.speed * 2) {
                if ((this.path = Game.lvl.objMap.getPath(this.position, this.basePosition, this.basePosition, this.moveRange + this.attackRange)).length == 0) {
                    if ((this.path = Game.lvl.objMap.getPath(this.position, this.basePosition)).length == 0)
                        this.path = [Game.lvl.objMap.getTile(this.basePosition)]
                }
            } else if (lng.getLength() > this.speed) {
                this.animation = Game.lvl.pool.switchAnimation(this.animations['run'], this.animation);
                this.position.add(lng.normalize().multiply(this.speed));
                this.animation.setDirection(lng.getDirection());
            } else {
                this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
                break;
            }
        }
    }
};

Friendly.prototype.del  = function () {
	if (this.moveRange > 0)
		this.cluster.release();

    if (this.tile.wObjects.indexOf(this) != -1)
	    this.tile.wObjects.splice(this.tile.wObjects.indexOf(this), 1);
	this.toDelete = true;
};

Friendly.prototype.checkMove = function (){
	var newTile =  Game.lvl.objMap.getTile(this.position);
	if (newTile != this.tile){
		this.tile.wObjects.splice(this.tile.wObjects.indexOf(this), 1);
		this.tile = newTile;
		this.tile.wObjects.push(this);
	}
};

Friendly.prototype.applyDamage = function (damage, source) {
	if (!this.hitPointsEffect)
		this.hitPointsEffect = new hitPointsEffect(this, true);

	if ((source)&&(source.penetration)){
		this.hitPoints -= Math.floor(damage*(1 - this.armor*(1-source.penetration)));
	} else {
		this.hitPoints -= Math.floor(damage*(1 - this.armor));
	}

	if (this.hitPoints <= 0) {
		if (!this.alreadyDead) {
            this.onDeath();
            this.soundDeath && this.soundDeath.play();
            if (source && source.base && source.base.n == 'necromantBoss') {
                if (this.attackRange < 128)
                    Game.lvl.objEnemy.push(new Enemy(Game.enemyList.redSkeleton, this.position.x, this.position.y, 'TownHall', 1));
                else
                    Game.lvl.objEnemy.push(new Enemy(Game.enemyList.skeletonArcher, this.position.x, this.position.y, 'TownHall', 1));
            }
        }
		this.alreadyDead = true;
	}
};


Friendly.prototype.applyDamageHero = function (damage, source, withoutSound) {
    if (!this.hitPointsEffect)
        this.hitPointsEffect = new hitPointsEffect(this, true);

    if (source && source.base && source.base.boss) {
        damage *= 0.5;
    }

    if ((source)&&(source.penetration)){
        this.hitPoints -= Math.floor(damage*(1 - this.armor*(1-source.penetration)));
    } else {
        this.hitPoints -= Math.floor(damage*(1 - this.armor));
    }

    if (this.hitPoints <= 0) {
        if (!this.alreadyDead) {
            this.deathCounter++;
            !withoutSound && this.soundDeath && this.soundDeath.play();
            this.onDeath();
        }
        this.alreadyDead = true;
    }
};

Friendly.prototype.scanEnemyAround = function () {

    if (this.basePositionMoved) {
        this.target = null;
        return;
    }

	if (this.target)
		if(this.target.hitPoints > 0 && this.target.invulnerable <= 0 && (this.scanTimeOut > 0 || !this.target.target || this.target.target == this))
			return;
		else
			this.path = [];

	this.target = null;

	if (this.scanTimeOut > 0)
		return;

	this.cluster.update();
	this.scanTimeOut = this.base.timeOut/2;
	var length = this.moveRange + this.attackRange;
	for (var tile in this.cluster.tiles){
		if (this.cluster.tiles[tile].isWalkable)
			for (var enemy in this.cluster.tiles[tile].parentTile.eObjects){
				if (this.cluster.tiles[tile].parentTile.eObjects[enemy].hitPoints > 0 && this.cluster.tiles[tile].parentTile.eObjects[enemy].invulnerable <= 0 &&((this.base.canAttackFly)||(!this.cluster.tiles[tile].parentTile.eObjects[enemy].flying))){
					var eLength = this.cluster.tiles[tile].parentTile.position.clone().sub(this.basePosition).getLength();
					if (eLength < length || (this.target && this.target.target && this.target.target === this)){
						length = eLength;
						this.target = this.cluster.tiles[tile].parentTile.eObjects[enemy];
					}
				}
			}
	}
};

Friendly.prototype.scanEnemyAroundArcher = function () {
    var tile;
	if (this.target && this.target.hitPoints > 0 &&  this.target.invulnerable <= 0 && this.target.position.clone().sub(this.position).getLength() < this.attackRange )
			return;

	this.target = null;

	if (this.scanTimeOut > 0)
		return;

	this.scanTimeOut = this.base.timeOut/2;
	var length = this.attackRange;

	for (var x = this.tile.positionInArray.x - parseInt(this.attackRange/64) - 1; x <= this.tile.positionInArray.x + parseInt(this.attackRange/64) + 1; x ++)
		for (var y = this.tile.positionInArray.y - parseInt(this.attackRange/64) - 1; y <= this.tile.positionInArray.y + parseInt(this.attackRange/64) + 1; y ++)
			if ((tile = Game.lvl.objMap.getNodeAt(x,y)))
				for (var enemy in tile.eObjects) {
					if (tile.eObjects[enemy].hitPoints > 0 && tile.eObjects[enemy].invulnerable <= 0){
						var eLength = tile.eObjects[enemy].position.clone().sub(this.basePosition).getLength();
						if (eLength < length){
							length = eLength;
							this.target = tile.eObjects[enemy];
						}
					}
				}
};

Friendly.prototype.strategyArcher = function (){
	var direction;
	this.timeOut -= Game.cycleTickTime*Game.speed;
	if (this.timeOut > 0)
		return;

	this.scanTimeOut -= Game.cycleTickTime*Game.speed;
	if (this.basePosition.clone().add(this.tileSdvig).sub(this.position).getLength() < this.speed*2) {
		this.scanEnemyAround();
		if (this.target) {
			direction = this.target.position.clone().sub(this.position);
			this.animation.setDirection(direction.getDirection());
			this.animation = Game.lvl.pool.switchAnimation(this.animations['fight'][this.fightAnim], this.animation);
			direction = this.position.clone().sub(this.target.position);
			direction.sub(direction.clone().normalize().multiply(32));
            this.soundShoot && this.soundShoot.play();
			Game.lvl.objEffect.push(new Arrow(this.position.clone().add(new Vec2(0, -32)), this.target, Game.sprites.arrow, 1, direction.getLength(), false, function(){
                this.target.applyDamage(this.parent.damage, this.parent);
            }, null, this));
			this.timeOut = this.base.timeOut;
		} else {
			this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
		}
	} else {
		if (this.path.length > 0)
			this.move();
		else if ((direction = this.basePosition.clone().add(this.tileSdvig).sub(this.position)).getLength() > this.speed*2) {
			if ((this.path = Game.lvl.objMap.getPath(this.position, this.basePosition, this.basePosition, this.moveRange + this.attackRange)).length == 0){
				if ((this.path = Game.lvl.objMap.getPath(this.position, this.basePosition)).length == 0)
					this.path = [Game.lvl.objMap.getTile(this.basePosition)]
			}
		} else {
			this.timeOut = this.base.timeOut;
			this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
		}
	}
};

Friendly.prototype.strategy = function (){
	var direction;
	this.timeOut -= Game.cycleTickTime*Game.speed;
	this.scanTimeOut -= Game.cycleTickTime*Game.speed;

	if (this.timeOut > 0)
		return;

	this.scanEnemyAround();

	if (this.target && !this.basePositionMoved && (this.basePosition.clone().sub(this.position).getLength() < (this.moveRange + this.attackRange))) {
        direction = this.target.position.clone().sub(this.position);
		if (direction.getLength() < this.attackRange + this.target.hitRange) {
            if (this.isHero && this.abilityTrain){
                this.abilityTrain = false;
                Game.lvl.objGui.push(new Gui(Game.guiList.heroCast));
                Game.lvl.paused = true;
            }
			this.animation.setDirection(direction.getDirection());
            this.soundCC && this.soundCC.play();
			this.animation = Game.lvl.pool.switchAnimation(this.animations['fight'][this.fightAnim], this.animation);
			this.timeOut = this.base.timeOut;
			this.target.applyDamage(this.damage, this);
		} else {
			if ((this.path.length > 0) && (this.path[0].position.clone().sub(this.target.position).getLength()<this.attackRange)){
				this.move();
			} else if ((this.target.position.clone().sub(this.basePosition).getLength()>this.attackRange + this.moveRange)||((this.path = Game.lvl.objMap.getPath(this.position, this.target.position, this.basePosition, this.moveRange + this.attackRange)).length == 0)){
				this.target = null;
				this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
			} else
				this.animation.setDirection(this.path[this.path.length -1].position.clone().sub(this.position).getDirection());
		}
	} else {
		if ((direction = this.basePosition.clone().add(this.tileSdvig).sub(this.position)).getLength() > this.speed*2) {
            this.move();
		} else {
             if (this.basePositionMoved) {
                 this.basePositionMoved = false;
                 this.timeOut = 0;
                 this.scanTimeOut = 0;
             }
             this.timeOut = this.base.timeOut;
             this.animation = Game.lvl.pool.switchAnimation(this.animations['idle'], this.animation);
		}
	}
};