﻿GameLogic =  function(waves, openHero, money, hitPointsCounter) {
    this.waves = JSON.parse(JSON.stringify(waves));
    this.lvlHitPoints = 0;
    var self = this;
    var bounty = 0;
    this.wavesCount = 0;
    this.currentWave = 1;
    waves.forEach(function(wave){
        if (wave.spawners.length > 0)
            self.wavesCount++;
        wave.spawners.forEach(function(spawner){
            spawner.spawns.forEach(function(spawn){
                self.lvlHitPoints += Game.enemyList[spawn.name].hitPoints * spawn.count;
                bounty += Game.enemyList[spawn.name].bounty * spawn.count
            })
        })
    });
    this.startResource = money;
    var addMoney = 0;
    switch (Game.globallogic.getTownHallLvl()) {
        case 1:
            addMoney = 100;
            break;
        case 2:
            addMoney = 200;
            break;
        case 3:
            addMoney = 400;
            break;
        case 4:
            addMoney = 600;
            break;
    }
    this.startResource += addMoney;
    this.hitPointsCounter = hitPointsCounter;
    this.winsound = new MyAudio('Stinger_15', false, 0.4);
    this.losesound = new MyAudio('Stinger_8', false, 0.4);
    this.horn = Game.lvl.audioPool.get('horn', 'Horn_1', 0.4);
    this.init.call(this);
};

GameLogic.prototype.sellUnit = function(target) {
    target.hitPoints = 0;
    target.alreadyDead = true;
    target.onDeath();
};

GameLogic.prototype.sellBuilding = function(target) {
    Game.lvl.audioPool.get('Gold', 'Gold', 0.3, 2).play();
    var currentItem = target.basicParams;
    var sellCost = currentItem.cost / 2;
    while (currentItem.children) {
        currentItem = Game.buildingsList[currentItem.children];
        sellCost += currentItem.cost / 2;
    }
    Game.lvl.gameLogic.harvestMoney(parseInt(sellCost), target.position.clone().add(new Vec2(0, -32)));
    Game.events.emit('sellTower');
    target.onDeath();
};


GameLogic.prototype.upgradeBuilding = function (target, tower, x, y) {
    Game.lvl.audioPool.get('build', 'Build', 0.3, 2).play();

    if (localStorage.getItem("tutor.upgrade") != "true") {
        localStorage.setItem("tutor.upgrade", "true");
        log("tutor.upgrade", 0);
    }
    Game.events.emit('towerBuild');
    target.del();
    Game.lvl.gameLogic.build(tower, target.position.x, target.position.y);
    Game.lvl.objEffect.push(new Effect(target.position.clone(), new Vec2(0,0), Game.lvl.objEffect, Game.animations.LVLUP, Game.animations.LVLUP.playTime, 1.5, null, false, false))
};

GameLogic.prototype.upgradeUnit = function (target, targetUnit, x, y) {
    target.onDeath();
    target.del();
    Game.lvl.gameLogic.hire(targetUnit, x, y);
};

GameLogic.prototype.moveUnit = function (target, x, y) {
    target.movePosition(new Vec2(x, y));
};

GameLogic.prototype.init = function() {
    this.crazy = true;
    this.loyalist = true;
    this.horde = true;
    this.loneWolf = true;
    this.lazarus = true;
    this.whereIsEverybody = true;

    this.passedTime = 0;
    this.wave = 1;
    this.currectWave =  0;
    this.allWaves = 0;
    this.initWavesIcons();
    for (var i =0; i< this.waves.length; i++){
        if (this.waves[i].spawners.length > 0){
            this.allWaves ++;
        } else {
            break;
        }
    }
    this.hero = null;
    this.state = "day";
    Game.dayTime = 45000;
    this.timeOut = Game.dayTime;
    this.resource = this.startResource;
    this.allLimit = 0;
    this.limit = 0;
    this.spawTagCounter = 1;
    this.onDayCallBack =[];
    this.onNightCallBack =[];
    this.buildList =[];
    this.hireList = [];

    if (Game.menu.selectedLvl === "0") {
        this.state = 'tutorBuild';
        this.timeOut = 0;
        this.ballon = new MessageBalloon(Game.strings[Game.language].tutorBuild, new Vec2(1376, 580), true, Game.lvl.gameContainer);
        setTimeout(function(){
            Game.lvl.pausedUpdate.push(new Gui(Game.guiList.messageTowers));
        },1)
    } else if (Game.menu.selectedLvl === "1") {
        setTimeout(function(){
            this.build(Game.buildingsList.house, 800, 1056);
            Game.lvl.pausedUpdate.push(new Gui(Game.guiList.messageTowers2));

        }.bind(this),1);
    }
};

GameLogic.prototype.initWavesIcons = function () {
    var hasIcons = false, iter;
    for (iter = this.waves[this.currectWave].spawners.length - 1; iter>=0; iter--) {
        if (this.waves[this.currectWave].spawners[iter].isIcon) {
            hasIcons = true;
            break;
        }
    }

    for (iter = this.waves[this.currectWave].spawners.length - 1; iter>=0; iter--) {
        if (this.waves[this.currectWave].spawners[iter].isIcon || !hasIcons)
            new WaveIconEffect(this.waves[this.currectWave].spawners[iter]);
    }
};

GameLogic.prototype.update = function(){
    var spawner, spawn;
    this.timeOut -= Game.cycleTickTime*Game.speed;

    this.ballon && this.ballon.parental && (this.ballon.container.position = this.ballon.parental.container.position.clone().sub(new Vec2(48, 0)));

    if (Game.menu.selectedLvl == "0" && this.wave == 1) {
        this.hero.applyDamage(10000000000, null, true);
        this.hero.deadTimer = 200;
        this.hero.animation.alpha = 0;
    }

    if (this.timeOut < 0){
        this.onTimeOut();
    }

    if (this.state ==='night') {
        Game.fromLvlBegin += Game.cycleTickTime*Game.speed;
        this.passedTime += Game.cycleTickTime*Game.speed;
        for (var iter = this.waves[this.currectWave].spawners.length - 1; iter>=0; iter--) {
            spawner = this.waves[this.currectWave].spawners[iter];
            if (spawner) {
                if (!spawner.tag)
                    spawner.tag = this.spawTagCounter++;
                if (isNaN(spawner.cd)) {
                    if (spawner.spawns[0]) {
                        spawner.cd = parseInt(spawner.spawns[0].time);
                    } else
                        spawner.cd = 0;
                }

                spawn = spawner.spawns[0];
                if (!spawn) {
                    this.waves[this.currectWave].spawners.splice(this.waves[this.currectWave].spawners.indexOf(spawner), 1);
                } else if (spawner.cd <= 0) {
                    Game.events.emit(Game.profile +'enemySpawned:'+spawn.name);
                    Game.lvl.objEnemy.push(new Enemy(Game.enemyList[spawn.name], spawner.x, spawner.y, 'townHall', spawner.tag));
                    if (--spawn.count <= 0) {
                        spawner.spawns.shift();
                        if (spawner.spawns.length > 0) {
                            spawner.cd += parseInt(spawner.spawns[0].time);
                            spawner.cd -= Game.cycleTickTime*Game.speed;
                        } else {
                            this.waves[this.currectWave].spawners.splice(this.waves[this.currectWave].spawners.indexOf(spawner), 1);
                        }
                    } else {
                        spawner.cd += parseInt(spawn.rTime);
                        spawner.cd -= Game.cycleTickTime*Game.speed;
                    }
                } else {
                    spawner.cd -= Game.cycleTickTime*Game.speed;
                }
            }
        }
    } else if ((this.state == 'won')||(this.state == 'lose')) {
        this.timeOut = 0;
    }
};

GameLogic.prototype.moveHero = function(position) {
    if (this.hero)
        if (this.hero.hitPoints > 0) {
            position = Game.lvl.objMap.getTile(position).position;
            this.hero.target = null;
            this.hero.path = [];
            this.hero.basePosition = Game.lvl.objMap.getClosesEmptyTile(position).position;
            this.hero.initCluster();
            this.hero.timeOut = 0;
        }
};

GameLogic.prototype.onTimeOut =  function() {
    if (this.state === 'tutorBuild') {
        if (Game.lvl.objBuildings.length > 1) {
            log('tutor.build', 0);
            console.log('start');
            this.state = 'tutorStart';
            Game.lvl.pool.pull(this.ballon.container);
            var waveIcon =  Game.lvl.objEffect.filter(function(eff){return eff.isWaveIcon}).pop();
            if (waveIcon)
                this.ballon = new MessageBalloon(Game.strings[Game.language].tutorStart, waveIcon.container.position.clone(), false, Game.lvl.effectContainer);
            this.ballon.parental = waveIcon;
        }
    } else if (this.state === "day" && (this.currectWave != 0 || Game.multiplayerLvl)) {
        this.night();
    } else if (this.state === "night") {
        this.currectSpawn = 0;
        this.passedTime = 0;
        while ((this.currectWave<this.waves.length)&&(this.waves[this.currectWave].spawners.length === 0))
            this.currectWave ++;

        if (this.currectWave >= this.waves.length) {
            if (Game.lvl.objEnemy.length == 0)
                this.won();
            else {
                this.currectWave--;
                this.timeOut = 5000;
            }
        } else
            this.day();

    }
};

GameLogic.prototype.build = function(tower, x, y) {
    Game.lvl.audioPool.get('build', 'Build', 0.3, 2).play();

    Game.events.emit('towerBuild');
    this.resource -= tower.cost;
    this.loneWolf = false;

    if (this.loyalist && (this.loyalist === true || this.loyalist === tower.subtype) && tower.subtype !== ' house') {
        this.loyalist = tower.subtype;
        this.whereIsEverybody = false;
    } else {
        this.loyalist = false;
    }
    if (this.horde && tower.subtype !== ' house')
        this.horde = false;

    Game.lvl.objBuildings.push(new Building(tower, x, y));
};

GameLogic.prototype.hire = function(unit, x, y){
    var tile = Game.lvl.objMap.getClosesEmptyTile(new Vec2(x,y));
    if (tile){
        this.resource -= unit.cost;
        this.limit += unit.limitCost;
        Game.lvl.objFriendly.push(new Friendly(unit, tile.position.x, tile.position.y));
    }
};

GameLogic.prototype.night = function () {
    this.horn.play();

    if (this.currectWave != 0)
        this.wave++;

    if(this.ballon && Game.menu.selectedLvl == "0") {
        log('tutor.wave', 0);
        Game.lvl.pool.pull(this.ballon.container);
        this.ballon = null;
        Game.lvl.timerGui.ballon = new MessageBalloon(Game.strings[Game.language].tutorSpeed, Game.lvl.timerGui.buttonsprite.position, false, Game.lvl.guiContainer);
    } else if (this.ballon && Game.menu.selectedLvl == "1") {
        Game.lvl.pool.pull(this.ballon.container);
        this.ballon = null;
        log('tutor.early', 0);
    }

    if (Game.menu.selectedLvl === "0" && this.wave === 2) {
        this.hero.animation.alpha = 1;
        Game.lvl.gameLogic.hero.deadTimer = 0;
        Game.lvl.gameLogic.hero.hitPoints = Game.lvl.gameLogic.hero.maxHitPoints;
        Game.lvl.gameLogic.hero.alreadyDead = false;
        Game.lvl.gameLogic.hero.timeOut = -1;
        Game.lvl.gameLogic.hero.movePosition(Game.lvl.gameLogic.hero.basePosition, true);
        Game.lvl.gameLogic.hero.movePosition(Game.lvl.gameLogic.hero.basePosition, true);
        Game.lvl.gameLogic.hero.animation = Game.lvl.pool.switchAnimation(Game.lvl.gameLogic.hero.animations['idle'], Game.lvl.gameLogic.hero.animation);
        setTimeout(function(){
            Game.lvl.objGui.push(new Gui(Game.guiList.heroMove));
            Game.lvl.paused = true;
        }, 1);
        //this.ballon = new MessageBalloon(Game.strings[Game.language].tutorHero,  Game.lvl.gameLogic.hero.position.clone(), false, Game.lvl.effectContainer);
        //this.ballon.balloon.scale.set(0.7, 0.7);
        //var sprite = Game.lvl.pool.getSprite(Game.sprites.heroIdle0);
        //sprite.anchor.set(0.5, 0.5);
        //sprite.scale.set(1.5, 1.5);
        //
        //Game.lvl.objGui.push(new Gui(Game.guiList.messageIcon).setMessage(Game.strings[Game.language].heroHintMessage).setTitle(Game.strings[Game.language].heroHintTitle).setImage(sprite).setOnClose());
    } else if (false && Game.menu.selectedLvl == "1" && this.wave == 2 && Game.buildingsList.orcHouse.isOpen) {

        var container = new PIXI.Container();
        var sprite = Game.lvl.pool.getSprite(Game.sprites.tent);

        sprite.anchor.set(0.5, 0.5);
        container.addChild(sprite);

        container.addChild(Game.lvl.pool.getSprite(Game.sprites.RadialGui_A));
        var icon = Game.lvl.pool.getSprite(Game.sprites['RadialGui_B']), pos = new Vec2(0,0);
        icon.scale = new Vec2(1.3, 1.3);
        icon.position = new Vec2(-120, 120);
        container.addChild(icon);
        icon = Game.lvl.pool.getSprite(Game.sprites['upgSellIcon']);
        icon.position = new Vec2(-120, 120);
        container.addChild(icon);

        pos = new Vec2(120, 120);
        icon = Game.lvl.pool.getSprite(Game.sprites['RadialGui_B']);
        icon.position = pos;
        icon.scale = new Vec2(1.3, 1.3);
        container.addChild(icon);
        icon = Game.lvl.pool.getSprite(Game.sprites['upgFlagIcon']);
        icon.position = pos;
        container.addChild(icon);

        pos = new Vec2(-120, -120);
        icon = Game.lvl.pool.getSprite(Game.sprites['RadialGui_B']);
        icon.position = pos;
        icon.scale = new Vec2(1.3, 1.3);
        container.addChild(icon);
        icon = Game.lvl.pool.getSprite(Game.sprites['uorcWarrior']);
        icon.position = pos;
        container.addChild(icon);
        var price = new SimpleFont(Game.buildingsList.orcHouse.cost, 'white');
        price.container.position = pos.clone().sub(new Vec2(('190').length * 12, -24));
        price.container.scale.set(0.5 ,0.5);
        container.addChild(price.container);


        pos = new Vec2(120, -120);
        icon = Game.lvl.pool.getSprite(Game.sprites['RadialGui_B']);
        icon.position = pos;
        icon.scale = new Vec2(1.3, 1.3);
        container.addChild(icon);
        icon = Game.lvl.pool.getSprite(Game.sprites['upgLockedIcon']);
        icon.position = pos;
        container.addChild(icon);
        container.scale.set(0.8, 0.8);

        if (this.state !='sell')
            icon = Game.lvl.pool.getSprite(Game.sprites['upgSellIcon']);
        Game.lvl.objGui.push(
            new Gui(Game.guiList.tipsIcon)
                .setTitle(Game.strings[Game.language].tutorUpgradeTitle)
                .setImage(container)
                .setDescription(Game.strings[Game.language].tutorUpgradeText)
        );
    }

    var key;
    for (var iter = Game.lvl.objEffect.length - 1; iter>=0; iter--) {
        if (Game.lvl.objEffect[iter].isWaveIcon) {
            Game.lvl.objEffect[iter].del();
            Game.lvl.objEffect.splice(Game.lvl.objEffect.indexOf(Game.lvl.objEffect[iter]), 1);
        }
    }

    for ( key in this.onNightCallBack)
        this.onNightCallBack[key]();
    for ( key in Game.lvl.objFriendly)
        Game.lvl.objFriendly[key].onNight();

    if (this.timeOut > 2000 && this.currectWave != 0 && !Game.multiplayerLvl) { //TODO DEBUG
        Game.events.emit('calledEarly');
        this.resource += parseInt(this.timeOut/500)+parseInt(this.timeOut/1000);
        Game.lvl.objEffect.push(new moneyEffect(parseInt(this.timeOut/500), new Vec2(100, 150), true));
    } else {
        if (this.crazy && this.currectWave != 0)
            this.crazy = false;
    }
    this.timeOut = 1000;
    var spawn, localTimeOut = 0;
    for (var i in this.waves[this.currectWave].spawners) {
        localTimeOut = 0;
        for (var s in this.waves[this.currectWave].spawners[i].spawns) {
            localTimeOut += this.waves[this.currectWave].spawners[i].spawns[s].time;
            localTimeOut += this.waves[this.currectWave].spawners[i].spawns[s].rTime * (this.waves[this.currectWave].spawners[i].spawns[s].count-1);
        }
        if (this.timeOut < localTimeOut)
            this.timeOut = localTimeOut;
    }
    this.timeOut += 5000;
    this.state = 'night';
};


GameLogic.prototype.day = function (){
    var key;
    for (key in this.onDayCallBack)
        this.onDayCallBack[key]();
    for (key in Game.lvl.objBuildings)
        Game.lvl.objBuildings[key].onDay();
    for (key in Game.lvl.objFriendly)
        Game.lvl.objFriendly[key].onDay();

    if (this.hero)
        this.hero.onDay();
    this.hero.onDay();

    Game.speed = 1;
    this.initWavesIcons();
    if (Game.menu.selectedLvl != "0")
        this.timeOut = Game.dayTime;
    else
        this.timeOut = 7000;

    if (Game.menu.selectedLvl == "1" && this.currectWave == 1) {
        var waveIcon =  Game.lvl.objEffect.filter(function(eff){return eff.isWaveIcon}).pop();
        if (waveIcon) {
            setTimeout(function(){
                this.ballon = new MessageBalloon(Game.strings[Game.language].tutorEarly, waveIcon.container.position.clone(), false, Game.lvl.effectContainer);
                this.ballon.parental = waveIcon;
            }.bind(this))
        }
    }
    this.state = 'day';
    this.builds = 'builds';
};

GameLogic.prototype.won = function () {
    this.winsound.play();
    this.state ='won';
    var hitpoints = Game.lvl.objBuildings[0].hitPoints;
    Game.lvl.firstPassed = Game.globallogic.getLvlWithName(Game.menu.selectedLvl) &&  Game.globallogic.getLvlWithName(Game.menu.selectedLvl).passed == 0;

    var newStart = hitpoints > 18 ? 3 : hitpoints > 5 ? 2 : 1;
    log('won.'+Game.menu.selectedLvl, newStart);
    if (Game.globallogic.getLvlWithName(Game.menu.selectedLvl) && newStart > Game.globallogic.getLvlWithName(Game.menu.selectedLvl).passed)
        newStart = newStart - Game.globallogic.getLvlWithName(Game.menu.selectedLvl).passed;
    else
        newStart = 0;

    this.skulls = parseInt((this.lvlHitPoints*Game.globallogic.getHitpointsModificator()/10000) + newStart*75 + 50) || 0;
    Game.globallogic.addSkulls(this.skulls);
    if (Game.globallogic.getLvlWithName(Game.menu.selectedLvl)) {
        Game.globallogic.getLvlWithName(Game.menu.selectedLvl).setPassed(hitpoints > 18 ? 3 : (hitpoints > 5 || Game.menu.selectedLvl === "0") ? 2 : 1);
        Game.events.emit('lvlWin');
    }

    Game.noTouch = 2000;
    Game.speed = 1;
    for (var i = 1; i < 6; i++){
        (function(){
            for (var i2 = 0; i2 < myRandom(2,4); i2++) {
                setTimeout(function () {
                    var particle = 'Pinkfireworks';
                    switch (myRandom(0,5)){
                        case 0:
                            particle = 'Pinkfireworks';
                            break;
                        case 1:
                            particle = 'Greenfireworks';
                            break;
                        case 2:
                            particle = 'Mixedfireworks';
                            break;
                        case 3:
                            particle = 'Bluefireworks';
                            break;
                        case 4:
                            particle = 'Redfireworks';
                            break;
                    }
                    var config = clone(Game.particlesList[particle]);
                    config.pos.x = myRandom(0, 2048);
                    config.pos.y = myRandom(0, 1536);
                    var textures = [];
                    config.images.forEach(function (image) {
                        textures.push(PIXI.TextureCache[image])
                    });
                    this.ring = new cloudkid.Emitter(
                        Game.lvl.containerForParticle(config),
                        textures,
                        config
                    );
                    this.ring.emit = true;
                    Game.lvl.pausedUpdate.push(this.ring);
                }, i * 300 + i2*100);
            }
        })()
    }
    if (Game.menu.selectedLvl == "27")
        Game.lastSujet = true;

    setTimeout(function(){Game.lvl.objGui.push(new Gui(Game.guiList.won));}, 1000);
    setTimeout(function(){
        Game.lvl.paused = true;
    }.bind(this), 4000);
};

GameLogic.prototype.lose = function () {
    log('lose.'+Game.menu.selectedLvl, this.currectWave);
    this.losesound.play();
    this.state ='lose';
    Game.events.emit('lvlLose');
    var leftHp = 0;
    this.waves.forEach(function(wave){
        wave.spawners.forEach(function(spawner){
            spawner.spawns.forEach(function(spawn){
                leftHp += Game.enemyList[spawn.name].hitPoints * spawn.count;
            })
        })
    });
    this.skulls = parseInt(((this.lvlHitPoints-leftHp)*Game.globallogic.getHitpointsModificator()/10000) + 5) || 0;
    Game.globallogic.addSkulls(this.skulls);
    Game.noTouch = 2000;
    Game.lvl.objGui.push(new Gui(Game.guiList.lose));
    setTimeout(function(){
        Game.lvl.paused = true;
    }.bind(this), 2000);
};

GameLogic.prototype.addMoney = function(money) {
    if (this.hero && this.hero.hitPoints > 0)
        this.hero.addExperience(money);
    this.resource += money;
};

GameLogic.prototype.harvestMoney = function (money, position) {
    this.resource += money;
    Game.lvl.objEffect.push(new moneyEffect(money, position));
};

