﻿/**
 * Created by mnill on 23.08.14.
 */


GlobalLogic =  function() {
    this.skulls = 0;
    this.availableStars = 0;
    this.allStars = 0;
    this.lvls = [];
    this.achievements = [];
    this.heroes = [];
    this.upgrades = [];
    this.premium = [];
    this.selectedHero = '';
    this.mapPosition = new Vec2(0,0);
    this.difficulty = localStorage.getItem('difficulty') || 'normal';
    this.soundEnable = localStorage.getItem('audio') !== 'false';
    this.musicEnable = localStorage.getItem('music') !== 'false';
    this.init.call(this);
    this.loadPurchase();
};

GlobalLogic.prototype.toggleSound = function() {
    this.soundEnable = !this.soundEnable;
    localStorage.setItem('audio', this.soundEnable);
};

GlobalLogic.prototype.showAchievements = function(){
    var dialog = new Cocoon.Widget.WebDialog();
    dialog.show("achievements.html");
    var achs = [];
    for (var key in Game.LOCAL_ACHIEVEMENTS_MAP)
        if (Game.LOCAL_ACHIEVEMENTS_MAP.hasOwnProperty(key))
            achs.push(Game.LOCAL_ACHIEVEMENTS_MAP[key].toJSON());
    var js = "addAchievements(" + JSON.stringify(achs) + ");";
    dialog.eval(js);
};

GlobalLogic.prototype.showLibrary = function(){
    var dialog = new Cocoon.Widget.WebDialog();
    dialog.show("library.html");
    var enemies = [];
    for (var key in Game.enemyList){
        var enemy = Game.enemyList[key];
        if (Game.enemyList.hasOwnProperty(key) && key != 'ogreBoss' && key != 'dragonBoss' && key != 'necromantBoss') {
            var unlocked = localStorage.getItem(Game.profile + 'summon:' + enemy.n);
            if (unlocked) {
                var description = Game.strings[Game.language].monsters[enemy.n].special;
                description = description.replace('\r', '<br>').replace('\r', '<br>').replace('\r', '<br>');
                enemies.push({
                    unlocked: true,
                    imageURL: "unitsTips/" + enemy.n + '.png',
                    name: Game.strings[Game.language].monsters[enemy.n].name,
                    description: description,
                    damage: parseInt(enemy.damage * 1000 / enemy.timeOut),
                    hp: enemy.hitPoints,
                    bounty: enemy.bounty,
                    armor: enemy.armor,
                    magicArmor: enemy.magicArmor
                })
            } else {
                enemies.push({
                    name: 'Unknown',
                    description: '',
                    damage: '?',
                    hp: '?',
                    bounty: '?',
                    armor: '?',
                    magicArmor: '?'
                })
            }
        }
    }
    var js = "addUnits(" + JSON.stringify(enemies) + ");";
    dialog.eval(js);
};

GlobalLogic.prototype.setDifficulty = function(difficulty) {
    this.difficulty = difficulty;
    localStorage.setItem('difficulty', difficulty);
};

GlobalLogic.prototype.getHitpointsModificator = function() {
    switch (this.difficulty) {
        case 'hard':
            return 1.3;
            break;
        case 'easy':
            return 0.9;
        default:
            return 1;
            break;
    }
};

GlobalLogic.prototype.toggleMusic = function() {
    this.musicEnable = !this.musicEnable;
    localStorage.setItem('music', this.musicEnable);
    if (!this.musicEnable)
        Game.currentPlayedMusic && Game.currentPlayedMusic.stop();
    else
        Game.currentPlayedMusic && Game.currentPlayedMusic.play();
};

GlobalLogic.prototype.init = function() {
    this.initUpgrades();
    this.initLvls();
    this.initHeroes();
};

GlobalLogic.prototype.resetUpgrades = function() {
    this.availableStars = this.allStars;
    this.upgrades.forEach(function (upgrade){
        upgrade.reset();
    })
};

GlobalLogic.prototype.save = function() {
    localStorage.setItem(Game.profile + 'allStars', this.allStars);
    this.upgrades.forEach(function(upgrade){
        upgrade.save();
    });
};

GlobalLogic.prototype.load = function() {
    this.lvls.forEach(function(lvl){
        lvl.load();
    });
    this.availableStars = this.allStars;
    this.heroes.forEach(function(hero){
        hero.load();
    });
    this.upgrades.forEach(function(upgrade) {
        upgrade.load();
    });

    var mapX = parseInt(localStorage.getItem(Game.profile + 'mapX')) || 0;
    var mapY = parseInt(localStorage.getItem(Game.profile + 'mapY')) || 0;
    this.mapPosition.set(mapX, mapY);

    var name = localStorage.getItem(Game.profile + 'selectedHero');
    if (name)
        this.selectedHero = this.getHeroWithName(name);
    else
        this.selectedHero = this.getHeroWithName('warrior');

    if (!this.selectedHero) {
        this.selectedHero = this.getHeroWithName('warrior');
    }

    this.goldBag = parseInt(localStorage.getItem(Game.profile + 'goldBag')) || 0;
    this.bomb = parseInt(localStorage.getItem(Game.profile + 'bomb')) || 0;
    this.iceStaff = parseInt(localStorage.getItem(Game.profile + 'iceStaff')) || 0;
    this.meteorite = parseInt(localStorage.getItem(Game.profile + 'meteorite')) || 0;
    this.life = parseInt(localStorage.getItem(Game.profile + 'life')) || 0;
    this.resurrection = parseInt(localStorage.getItem(Game.profile + 'resurrection')) || 0;

    this.skulls = parseInt(localStorage.getItem(Game.profile + 'skulls')) || 0;
};

GlobalLogic.prototype.loadPurchase = function () {
    this.purchasePrice = {};
    this.purchaseList = [
        { id: 'com.mnillstone.orcscrusade.skullssmall', type: 'consumable' },
        { id: 'com.mnillstone.orcscrusade.skullsmedium', type: 'consumable' },
        { id: 'com.mnillstone.orcscrusade.skullsbig', type: 'consumable' },
        { id: 'com.mnillstone.orcscrusade.scrollspack', type: 'consumable' },
        { id: 'com.mnillstone.orcscrusade.heroassassin', type: 'durable' },
        { id: 'com.mnillstone.orcscrusade.heroogre', type: 'durable' },
        { id: 'com.mnillstone.orcscrusade.herorider', type: 'durable' },
        { id: 'com.mnillstone.orcscrusade.heromage', type: 'durable' },
        { id: 'com.mnillstone.orcscrusade.heroarcher', type: 'durable' },
        { id: 'com.mnillstone.orcscrusade.heropack', type: 'durable' }
    ];
    if (Game.isWindowsPhone) {
        Game.currentApp.loadListingInformationAsync().done(
            function (listing) {
                this.purchaseList.forEach(function (purchase) {
                    var product = listing.productListings.lookup(purchase.id);
                    if (product) {
                        purchase.price = product.formattedPrice;
                        this.purchasePrice[purchase.id] = product.formattedPrice;
                    }
                }.bind(this))
            }.bind(this),
            function () {
                console.log("LoadListingInformationAsync API call failed", "error");
            });

        Game.currentApp.getUnfulfilledConsumablesAsync().done(
            function (unfulfilledList) {
                unfulfilledList.forEach(function (product) {
                    if (isNotLocallyFinalize(product.productId, product.transactionId)) {
                        log('purchase.' + product.productId, 0);
                        localyFinalize(product.productId, product.transactionId);
                    }
                    finalize(product.productId, product.transactionId);
                });
            },
            function (error) {
                console.log("Unable to get unfulfilled consumables.", "sample", "error");
            });

        var licenseInformation = Game.currentApp.licenseInformation;
        this.heroes.forEach(function (hero) {
            if (hero.purchaseId) {
                if (licenseInformation.productLicenses.lookup(hero.purchaseId).isActive || licenseInformation.productLicenses.lookup('com.mnillstone.orcscrusade.heropack').isActive)
                    hero.purchase();
            }
        });
        //this.getHeroWithName(name).purchase()
    } else if (Cocoon.nativeAvailable) {
        this.purchaseList.push({ id: 'com.mnillstone.orcscrusade.episode2', type: 'durable' });
        this.purchaseList.push({ id: 'com.mnillstone.orcscrusade.episode3', type: 'durable' });

        var service = Cocoon.InApp;

        service.on("purchase",{
            start: function(){ console.log( "Purchasing products..." ); },
            complete: function(purchaseInfo) {
                console.log('purchased', JSON.stringify(purchaseInfo));
                localStorage.setItem('disableAds', "true");
                switch (purchaseInfo.productId) {
                    case 'com.mnillstone.orcscrusade.heroassassin': case 'com.mnillstone.orcscrusade.heroogre': case 'com.mnillstone.orcscrusade.herorider':case 'com.mnillstone.orcscrusade.heromage':case 'com.mnillstone.orcscrusade.heroarcher':
                        this.heroes.forEach(function(hero){
                            if (hero.purchaseId == purchaseInfo.productId){
                                hero.purchase();
                            }
                        }.bind(this));
                    break;
                    case 'com.mnillstone.orcscrusade.heropack':
                        this.heroes.forEach(function (hero) {
                            hero.purchase();
                        });
                        Game.globalHeroIcon && Game.lvl && Game.lvl.pool && Game.lvl.pool.pull(Game.globalHeroIcon);
                    break;
                    case 'com.mnillstone.orcscrusade.scrollspack': case 'com.mnillstone.orcscrusade.skullssmall':case 'com.mnillstone.orcscrusade.skullsmedium':case 'com.mnillstone.orcscrusade.skullsbig':
                        if (isNotLocallyFinalize(purchaseInfo.productId, purchaseInfo.transactionId))
                                localyFinalize(purchaseInfo.productId, purchaseInfo.transactionId);
                            finalize(purchaseInfo.productId, purchaseInfo.transactionId);
                        if (Game.globalPackIcon && Game.lvl && Game.lvl.pool)
                            Game.lvl.pool.pull(Game.globalPackIcon);
                        break;
                    case 'com.mnillstone.orcscrusade.episode2':
                        this.unlockEpisode(1);
                        Game.purchaseEpisodeCallback && Game.purchaseEpisodeCallback();
                        break;
                    case 'com.mnillstone.orcscrusade.episode3':
                        this.unlockEpisode(2);
                        Game.purchaseEpisodeCallback && Game.purchaseEpisodeCallback();
                        if (localStorage.getItem('episode3Skull') != "true"){
                            localStorage.setItem('episode3Skull', "true");
                            this.addSkulls(10000);
                        }
                        break;
                }
            }.bind(this),
            error: function(productId, err){
                console.log('Purchase error', JSON.stringify(err));
            }
        });

        service.initialize({
                autofinish: true
            },
            function(error){
                if(error){
                    console.log("Error initializing store: " + error);
                } else {
                    if (localStorage.getItem('restored') != "true")
                        service.restorePurchases(function(error) {
                            if (error){
                                console.log("Purchase restore error: " + error);
                            } else {
                                localStorage.setItem('restored', 'true');
                                console.log("Purchases restored");
                            }
                        });
                }
            });

        var ids = [];
        this.purchaseList.forEach(function(p){ids.push(p.id)});
        service.fetchProducts(ids, function(products, error){
            if(error) {
                console.log("Fetch products : " + error);
            }
            else {
                for (var i = 0; i < products.length; i++) {
                    //service.addProduct(products[i]);
                    this.purchaseList.forEach(function (purchase) {
                        if (purchase.id == products[i].productId) {
                            purchase.price = products[i].localizedPrice;
                            this.purchasePrice[purchase.id] = purchase.price;
                        }
                    }.bind(this));
                }
            }
        }.bind(this));

        Cocoon.InApp.setValidationHandler(function(receipt, productId, completion) {
            console.log(receipt);
            xhrPost('purchase_'+ (Game.platform == 0 ? 'ios' : 'android'), {receipt:JSON.parse(receipt), uid:Game.uid}, function(err){
                console.log("Custom validation: " + !err + " " + productId);
                completion(!err);
            });
             //call completion function with true param if validation succeeds
        });
    } else if (Game.isCongregate) {
        kongregateAPI.loadAPI(function (){
            console.log('api loaded');
            // Set the global kongregate API object
            Game.kongregate = kongregateAPI.getAPI();
            Game.kongregate.mtx.requestUserItemList(null, function (result) {
                console.log("User item list received, success: " + result.success, result.data);
                if( result.success ) {
                        //service.addProduct(products[i]);
                    result.data.forEach(function (purchase) {
                        switch (purchase.identifier) {
                            case 'com.mnillstone.orcscrusade.heroassassin': case 'com.mnillstone.orcscrusade.heroogre': case 'com.mnillstone.orcscrusade.herorider':case 'com.mnillstone.orcscrusade.heromage':case 'com.mnillstone.orcscrusade.heroarcher':
                            this.heroes.forEach(function(hero){
                                if (hero.purchaseId == purchase.identifier) {
                                    hero.purchase();
                                }
                            }.bind(this));
                            break;
                            case 'com.mnillstone.orcscrusade.heropack':
                                this.heroes.forEach(function (hero) {
                                    hero.purchase();
                                });
                                Game.globalHeroIcon && Game.lvl && Game.lvl.pool && Game.lvl.pool.pull(Game.globalHeroIcon);
                                break;
                            case 'com.mnillstone.orcscrusade.scrollspack': case 'com.mnillstone.orcscrusade.skullssmall':case 'com.mnillstone.orcscrusade.skullsmedium':case 'com.mnillstone.orcscrusade.skullsbig':
                            if (isNotLocallyFinalize(purchase.identifier, purchase.id))
                                localyFinalize(purchase.identifier, purchase.id);
                            finalize(purchase.identifier, purchase.id);
                            break;
                        }
                    }.bind(this));
                }
            }.bind(this));
            Game.kongregate.mtx.requestItemList([],function(items){
                for (var i = 0; i < items.length; i++) {
                    //service.addProduct(products[i]);
                    this.purchaseList.forEach(function (purchase) {
                        if (purchase.id == items[i].productId) {
                            purchase.price = items[i].localizedPrice;
                            this.purchasePrice[purchase.id] = purchase.price +' Kreds';
                        }
                    }.bind(this));
                }
            }.bind(this))
        });
    }
};

GlobalLogic.prototype.purchaseEpisode = function(n) {
    Cocoon.InApp.purchase('com.mnillstone.orcscrusade.episode'+(n+1), 1);
};

GlobalLogic.prototype.unlockEpisode = function(n) {
    localStorage.setItem('unlockedEpisode'+n, "true");
    localStorage.setItem('disableAds', "true");
};

GlobalLogic.prototype.isEpisodeUnlocked = function(n) {
    return true;
};

GlobalLogic.prototype.getPurchasePrice = function (number) {
    if (Game.isCongregate){
        return (number == 1 ? '20 Kreds' : (number == 2 ? '30 Kreds' : '40 Kreds'))
    } else if (this.purchasePrice[getSkullsId(number)]){
        return this.purchasePrice[getSkullsId(number)]
    } else {
        return (number == 1 ? '1,99$' : (number == 2 ? '2.99$' : '4.99$'))
    }
};

GlobalLogic.prototype.purchaseIap = function(id) {
    //Game.lvl.objGui[Game.lvl.objGui.length-1].prize
    allert(id);
};

function getSkullsId(number) {
    return number == 1 ? 'com.mnillstone.orcscrusade.skullssmall' : (number == 2 ? 'com.mnillstone.orcscrusade.skullsmedium' : 'com.mnillstone.orcscrusade.skullsbig')
}

function finalize(id, transactionId) {
    if (Game.isWindowsPhone) {
        Game.currentApp.reportConsumableFulfillmentAsync(id, transactionId).done(
            function (result) {
                switch (result) {
                    case Windows.ApplicationModel.Store.FulfillmentResult.succeeded:
                        break;
                    case Windows.ApplicationModel.Store.FulfillmentResult.nothingToFulfill:
                        break;
                    case Windows.ApplicationModel.Store.FulfillmentResult.purchasePending:
                        break;
                    case Windows.ApplicationModel.Store.FulfillmentResult.purchaseReverted:
                        break;
                    case Windows.ApplicationModel.Store.FulfillmentResult.serverError:
                        break;
                }
            },
            function (error) {
                console.log("You bought. There was an error when attempting to fulfill.");
            });
    } else if (Cocoon.nativeAvailable) {
        Cocoon.InApp.consume(id, 1, function(consumed, error) {
            if(error) {
                console.log("Error consuming: " + error);
            }
            else{
                console.log("Consumed items: " + consumed);
            }
        });
    } else if (Game.isCongregate) {
        console.log('use item', transactionId);
        Game.kongregate.mtx.useItemInstance(transactionId, function(result){console.log('used', result, id, transactionId)});
        //kongregateAPI.getAPI().mtx.purchaseItems(['com.mnillstone.orcscrusade.skullssmall'], function(result){console.log(result)})
    }
}
function localyFinalize(id, transtactionId) {
    var ids = localStorage.getItem("transaction" + id);
    if (!ids)
        ids = "";

    try {
        ids = ids.split(',');
    }
    catch (e) {
        ids = [];
    }
    ids.push(transtactionId);
    localStorage.setItem("transaction" + id, ids.join(','));

    if (id == getSkullsId(1))
        Game.globallogic.addSkulls(10000);
    if (id == getSkullsId(2))
        Game.globallogic.addSkulls(25000);
    if (id == getSkullsId(3))
        Game.globallogic.addSkulls(50000);

    if (id == 'com.mnillstone.orcscrusade.scrollspack') {
        Game.globallogic['goldBag']+= 10;
        Game.globallogic['bomb'] += 10;
        Game.globallogic['meteorite'] += 10;
        Game.globallogic['life'] += 10;
        Game.globallogic['resurrection'] += 10;
        Game.globallogic['iceStaff'] += 10;
        localStorage.setItem(Game.profile + 'goldBag', Game.globallogic['goldBag']);
        localStorage.setItem(Game.profile + 'bomb', Game.globallogic['bomb']);
        localStorage.setItem(Game.profile + 'iceStaff', Game.globallogic['iceStaff']);
        localStorage.setItem(Game.profile + 'meteorite', Game.globallogic['meteorite']);
        localStorage.setItem(Game.profile + 'life', Game.globallogic['life']);
        localStorage.setItem(Game.profile + 'resurrection', Game.globallogic['resurrection']);
    }
}
function isNotLocallyFinalize(id, transtactionId) {
    var ids = localStorage.getItem("transaction" + id);
    if (!ids)
        ids = "";

    try {
        ids = ids.split(',');
    }
    catch(e){
        ids = [];
    }
    return ids.indexOf(transtactionId);
}

GlobalLogic.prototype.buyPurchase = function (number) {
    if (Game.isWindowsPhone) {
        Game.currentApp.requestProductPurchaseAsync(getSkullsId(number)).done(
            function (purchaseResults) {
                if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.succeeded) {
                    log('purchase.' + getSkullsId(number), 0);
                    localyFinalize(getSkullsId(number), purchaseResults.transactionId);
                    finalize(getSkullsId(number), purchaseResults.transactionId);
                } else if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.notFulfilled) {
                    if (isNotLocallyFinalize(getSkullsId(number), purchaseResults.transactionId)) {
                        log('purchase.'+ getSkullsId(number), 0);
                        localyFinalize(getSkullsId(number), purchaseResults.transactionId);
                    }
                    finalize(getSkullsId(number), purchaseResults.transactionId);
                } else if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.notPurchased) {
                    allert('Not purchased');
                    //TODO mb
                }
            },
            function () {
                allert("Unable to buy");
            });
    }  else if (Cocoon.nativeAvailable) {
        Cocoon.InApp.purchase(getSkullsId(number), 1);
    } else if (Game.isCongregate) {
        Game.kongregate.mtx.purchaseItems([getSkullsId(number)], function(result){
            if (result.success){
                localyFinalize(getSkullsId(number), result.purchase_id);
                finalize(getSkullsId(number), result.purchase_id)
            }
        })
    }

};

GlobalLogic.prototype.addSkulls = function(amout) {
    if (!parseInt(amout))
        return;
    this.skulls += parseInt(amout);
    if (this.skulls < 0)
        this.skulls = 0;
    localStorage.setItem(Game.profile + 'skulls', this.skulls);
};


GlobalLogic.prototype.addItem = function(item){
    if (item != 'goldBag' && item != 'bomb' && item != 'iceStaff' && item != 'meteorite' && item != 'life' && item != 'resurrection')
        return;

    this[item]++;

    if (this[item] < 0)
        this[item] = 0;

    localStorage.setItem(Game.profile + item, this[item]);
};

GlobalLogic.prototype.getItemPrice = function(item) {
    switch (item){
        case 'goldBag':
            return 600;
            break;

        case 'bomb':
            return 200;
            break;

        case 'iceStaff':
            return 500;
            break;

        case 'meteorite':
            return 999;
            break;

        case 'life':
            return 1500;
            break;

        case 'resurrection':
            return 900;
            break;

        default:
            return 0;
            break
    }
};


GlobalLogic.prototype.useItem = function(item) {
    if (item != 'goldBag' && item != 'bomb' && item != 'iceStaff' && item != 'meteorite' && item != 'life' && item != 'resurrection')
        return;

    this[item]--;

    if (this[item] < 0)
        this[item] = 0;

    localStorage.setItem(Game.profile + item, this[item]);
};


GlobalLogic.prototype.createNew = function() {

};

GlobalLogic.prototype.purchase = function(key) {
    if (!key)
        throw new Error('argument error');

    if (key instanceof String)
        key = this.getUpgradeByName(key);

    if (this.availableStars - key.cost >= 0){
        key.purchase();
        this.availableStars -= key.cost;
    } else {
        //TODO сброс
    }
};

GlobalLogic.prototype.getUpgradeWithName = function (name) {
    var upgrade;
    for (var key in this.upgrades)
        if (this.upgrades[key].name === name || this.upgrades[key].legalName === name) {
            upgrade = this.upgrades[key];
            break;
        }
    if (name == 'wall')
        return {purchased:true};
    return upgrade;
};

GlobalLogic.prototype.getTownHallLvl = function () {
    for (var i = 4; i>0; i--) {
        if (this.getUpgradeWithName('TawnHall'+i).purchased)
            return i;
    }
    return 0;
};

GlobalLogic.prototype.getLvlWithName = function (name) {
    var lvl;
    for (var key in this.lvls)
        if (this.lvls[key].name == name) {
            lvl = this.lvls[key];
            break;
        }
    return lvl;
};

GlobalLogic.prototype.getHeroWithName = function (name) {
    var hero;
    for (var key in this.heroes)
        if (this.heroes[key].name == name) {
            hero = this.heroes[key];
            break;
        }
    return hero;
};


GlobalLogic.prototype.initLvls = function () {
    this.lvls.push(new LvlPrefs('0', 'desert', new Vec2(796, 676)));
    this.lvls.push(new LvlPrefs('1', 'desert', new Vec2(869, 660)));
    this.lvls.push(new LvlPrefs('2', 'desert', new Vec2(853, 593)));
    this.lvls.push(new LvlPrefs('3', 'desert', new Vec2(900, 540)));
    this.lvls.push(new LvlPrefs('4', 'desert', new Vec2(950, 600)));
    this.lvls.push(new LvlPrefs('5', 'desert', new Vec2(980, 542)));
    this.lvls.push(new LvlPrefs('6', 'desert', new Vec2(1041, 506)));
    this.lvls.push(new LvlPrefs('7', 'desert', new Vec2(990, 445)));
    this.lvls.push(new LvlPrefs('10', 'forest', new Vec2(1040,313)));
    this.lvls.push(new LvlPrefs('11', 'forest', new Vec2(1090,243)));
    this.lvls.push(new LvlPrefs('12', 'forest', new Vec2(1005,175)));
    this.lvls.push(new LvlPrefs('13', 'forest', new Vec2(880, 177)));
    this.lvls.push(new LvlPrefs('14', 'forest', new Vec2(830, 210)));
    this.lvls.push(new LvlPrefs('15', 'forest', new Vec2(860, 270)));
    this.lvls.push(new LvlPrefs('16', 'forest', new Vec2(790, 300)));
    this.lvls.push(new LvlPrefs('17', 'forest', new Vec2(805, 360)));
    this.lvls.push(new LvlPrefs('18', 'forest', new Vec2(766, 456)));
    this.lvls.push(new LvlPrefs('20', 'graveyard', new Vec2(605,430)));
    this.lvls.push(new LvlPrefs('21', 'graveyard', new Vec2(500,430)));
    this.lvls.push(new LvlPrefs('22', 'graveyard', new Vec2(420,360)));
    this.lvls.push(new LvlPrefs('23', 'graveyard', new Vec2(290,380)));
    this.lvls.push(new LvlPrefs('24', 'graveyard', new Vec2(255,440)));
    this.lvls.push(new LvlPrefs('25', 'graveyard', new Vec2(290,510)));
    this.lvls.push(new LvlPrefs('26', 'graveyard', new Vec2(280,590)));
    this.lvls.push(new LvlPrefs('27', 'graveyard', new Vec2(215,620)));
};

GlobalLogic.prototype.initUpgrades = function () {
    this.upgrades.push(new Upgrade('house1', 'house', 0, null, new Vec2(148, 580), 'b'));
    this.upgrades.push(new Upgrade('goblinArcher', 'goblinHouse', 2, this.getUpgradeWithName('house1'), new Vec2(80, 410), 'a'));
    this.upgrades.push(new Upgrade('goblinSniper', 'goblinHouse2', 3, this.getUpgradeWithName('goblinArcher'), new Vec2(80, 240), 'a'));
    this.upgrades.push(new Upgrade('goblinShaman', 'goblinHouse3', 4, this.getUpgradeWithName('goblinSniper'), new Vec2(80, 70), null));

    this.upgrades.push(new Upgrade('orcWarrior', 'orcHouse', 2, this.getUpgradeWithName('house1'), new Vec2(208, 410), 'a'));
    this.upgrades.push(new Upgrade('orcWolfRider', 'orcHouse2', 3, this.getUpgradeWithName('orcWarrior'), new Vec2(208, 240), 'a'));
    this.upgrades.push(new Upgrade('orcGuardian', 'orcHouse3', 4, this.getUpgradeWithName('orcWolfRider'), new Vec2(208, 70), null));

    this.upgrades.push(new Upgrade('tower1', 'tower', 0, null, new Vec2(413, 580), 'a'));
    this.upgrades.push(new Upgrade('tower2', 'tower2', 2, this.getUpgradeWithName('tower1'), new Vec2(413, 410), 'a'));
    this.upgrades.push(new Upgrade('tower3', 'tower3', 3, this.getUpgradeWithName('tower2'), new Vec2(413, 240), 'b'));
    this.upgrades.push(new Upgrade('hordeTower', 'hordeTower', 4, this.getUpgradeWithName('tower3'), new Vec2(349, 70), null));
    this.upgrades.push(new Upgrade('sniperTower', 'sniperTower', 4, this.getUpgradeWithName('tower3'), new Vec2(477, 70), null));

    this.upgrades.push(new Upgrade('TawnHall1', 'TownHall', 1, null, new Vec2(618, 580), 'a'));
    this.upgrades.push(new Upgrade('TawnHall2', 'TownHall2', 2, this.getUpgradeWithName('TawnHall1'), new Vec2(618, 410), 'a'));
    this.upgrades.push(new Upgrade('TawnHall3', 'TownHall3', 3, this.getUpgradeWithName('TawnHall2'), new Vec2(618, 240), 'a'));
    this.upgrades.push(new Upgrade('TawnHall4', 'TownHall4', 4, this.getUpgradeWithName('TawnHall3'), new Vec2(618, 70), null));
    //
    this.upgrades.push(new Upgrade('magicTower1', 'magicTower', 0, null, new Vec2(843, 580), 'a'));
    this.upgrades.push(new Upgrade('magicTower2', 'magicTower2', 2, this.getUpgradeWithName('magicTower1'), new Vec2(843, 410), 'a'));
    this.upgrades.push(new Upgrade('magicTower3', 'magicTower3', 3, this.getUpgradeWithName('magicTower2'), new Vec2(843, 240), 'b'));
    this.upgrades.push(new Upgrade('poisonTower', 'poisonTower', 4, this.getUpgradeWithName('magicTower3'), new Vec2(779, 70), null));
    this.upgrades.push(new Upgrade('lightningTower', 'lightningTower', 4, this.getUpgradeWithName('magicTower3'), new Vec2(907, 70), null));
    //
    this.upgrades.push(new Upgrade('catapulta1', 'catapulta', 0, null, new Vec2(1112, 580), 'a'));
    this.upgrades.push(new Upgrade('catapulta2', 'catapulta2', 2, this.getUpgradeWithName('catapulta1'), new Vec2(1112, 410), 'a'));
    this.upgrades.push(new Upgrade('catapulta3', 'catapulta3', 3, this.getUpgradeWithName('catapulta2'), new Vec2(1112, 240), 'b'));
    this.upgrades.push(new Upgrade('cannonTower', 'cannonTower', 4, this.getUpgradeWithName('catapulta3'), new Vec2(1048, 70), null));
    this.upgrades.push(new Upgrade('ballistaTower', 'ballistaTower', 4, this.getUpgradeWithName('catapulta3'), new Vec2(1176, 70), null));
};

GlobalLogic.prototype.initHeroes = function () {
    this.heroes = [];
    this.heroes.push(new Hero('warrior', null));
    this.heroes.push(new Hero('fireMage', 'com.mnillstone.orcscrusade.heromage'));
    this.heroes.push(new Hero('archer', 'com.mnillstone.orcscrusade.heroarcher'));
    this.heroes.push(new Hero('wolfRider', 'com.mnillstone.orcscrusade.herorider'));
    this.heroes.push(new Hero('ogre', 'com.mnillstone.orcscrusade.heroogre'));
    this.heroes.push(new Hero('templarAssassin', 'com.mnillstone.orcscrusade.heroassassin'));
};

GlobalLogic.prototype.buyHero = function (name) {
    if (Game.isWindowsPhone) {
        Game.currentApp.requestProductPurchaseAsync(this.getHeroWithName(name).purchaseId).done(
            function () {
                if (Game.currentApp.licenseInformation.productLicenses.lookup(this.getHeroWithName(name).purchaseId).isActive) {
                    this.getHeroWithName(name).purchase();
                } else {
                    allert('not purchased');
                }
            }.bind(this),
            function () {
                allert('Unable to buy');
            });
    } else if (Cocoon.nativeAvailable) {
        Cocoon.InApp.purchase(this.getHeroWithName(name).purchaseId, 1);
    } else if (Game.isCongregate){
        Game.kongregate.mtx.purchaseItems([this.getHeroWithName(name).purchaseId], function(result){
            if (result.success){
                this.getHeroWithName(name).purchase();
            }
        }.bind(this))
    }};

GlobalLogic.prototype.buyHeroPack = function (callback) {
    if (Game.isWindowsPhone) {
        Game.currentApp.requestProductPurchaseAsync('com.mnillstone.orcscrusade.heropack').done(
            function () {
                if (Game.currentApp.licenseInformation.productLicenses.lookup('com.mnillstone.orcscrusade.heropack').isActive) {
                    this.heroes.forEach(function (hero) {
                        hero.purchase();
                    });
                    callback && callback(true);
                } else {
                    allert('not purchased');
                    //TODO ERROR
                }
            }.bind(this),
            function () {
                allert('Unable to buy');
            });
    } else if (Cocoon.nativeAvailable) {
        Cocoon.InApp.purchase('com.mnillstone.orcscrusade.heropack', 1);
   } else if (Game.isCongregate) {
        Game.kongregate.mtx.purchaseItems([this.getHeroWithName(name).purchaseId], function(result) {
            if (result.success) {
                this.heroes.forEach(function (hero) {
                    hero.purchase();
                })
            }
        }.bind(this));
    }
};

GlobalLogic.prototype.buyScrollPack = function (callback) {
    if (Game.isWindowsPhone) {
        Game.currentApp.requestProductPurchaseAsync('com.mnillstone.orcscrusade.scrollspack').done(
            function (purchaseResults) {
                if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.succeeded) {
                    log('purchase.com.mnillstone.orcscrusade.scrollspack', 0);
                    localyFinalize('com.mnillstone.orcscrusade.scrollspack', purchaseResults.transactionId);
                    finalize('com.mnillstone.orcscrusade.scrollspack', purchaseResults.transactionId);
                    callback && callback(true);
                } else if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.notFulfilled) {
                    if (isNotLocallyFinalize('com.mnillstone.orcscrusade.scrollspack', purchaseResults.transactionId)) {
                        log('purchase.' + 'com.mnillstone.orcscrusade.scrollspack', 0);
                        localyFinalize('com.mnillstone.orcscrusade.scrollspack', purchaseResults.transactionId);
                    }
                    callback && callback(true);
                    finalize('com.mnillstone.orcscrusade.scrollspack', purchaseResults.transactionId);
                } else if (purchaseResults.status === Windows.ApplicationModel.Store.ProductPurchaseStatus.notPurchased) {
                    allert('Not purchased');
                    //TODO mb
                }
            },
            function () {
                allert("Unable to buy");
            });
    }  else if (Cocoon.nativeAvailable) {
        Cocoon.InApp.purchase('com.mnillstone.orcscrusade.scrollspack', 1);
    } else if (Game.isCongregate){
        Game.kongregate.mtx.purchaseItems(['com.mnillstone.orcscrusade.scrollspack'], function(result){
            if (result.success) {
                localyFinalize('com.mnillstone.orcscrusade.scrollspack', result.id);
                finalize('com.mnillstone.orcscrusade.scrollspack', result.id)
            }
        })
    }
};


GlobalLogic.prototype.setSelectedHero = function (hero) {
    //TODO покупка
    if (this.selectedHero != hero)
        this.selectedHero = hero;
    else
        this.selectedHero = null;

    localStorage.setItem(Game.profile + 'selectedHero', this.selectedHero ? this.selectedHero.name : null);
};

GlobalLogic.prototype.isUpgraded = function(name) {
    return this.getUpgradeWithName(name).purchased
};

GlobalLogic.prototype.isPriceGot = function(lvl) {
    return localStorage.getItem(Game.profile + 'lvlPrise'+lvl) == 'true';
};

GlobalLogic.prototype.getPrize = function (lvl) {
    if (lvl == "0" || lvl =="7" || lvl == "27" || lvl == "18") {
        return getRandomSkullsOrScrollPrize(true);
    } else if (lvl == "1" || lvl == "2" || lvl == "3" || lvl == "4" || lvl == "5") {
        return getRandomSkullsOrScrollPrize(false);
    } else {
        var lastPrize = parseInt(localStorage.getItem(Game.profile + 'lastprize')) || 0;
        if (--lastPrize <= 0) {
            localStorage.setItem(Game.profile + 'lastprize', myRandom(4,7));
            return getRandomOffPrize(parseInt(lvl) > 10, lvl)
        } else {
            localStorage.setItem(Game.profile + 'lastprize', lastPrize);
            return getRandomSkullsOrScrollPrize(false);
        }
    }
};

function getRandomOffPrize(withHeroChance, lvl) {
    var heroShowed = parseInt(localStorage.getItem(Game.profile+'lastprizehero')) || 0;
    var packShowed = parseInt(localStorage.getItem(Game.profile+'lastprizepack')) || 0;
    var heroesPurchased = 0;
    Game.globallogic.heroes.forEach(function(h){
        if (h.purchased)
            heroesPurchased++;
    });
    var container = new PIXI.DisplayObjectContainer();
    container.interactive = true;
    var text = new PIXI.Text('', {
        font: Game.getFont(48),
        align: 'center',
        fill: '#F0DC82'
    });
    text.anchor.set(0.5, 0.5);
    text.position.set(0, 0);

    if (((withHeroChance && heroShowed == 0 && lvl > 10) || (withHeroChance && heroShowed == 1 && lvl > 20)) && heroesPurchased < 4) {
        var yestochedheroes = false;
        localStorage.setItem(Game.profile+'lastprizehero', ++heroShowed);
        container.addChild(Game.lvl.pool.getSprite(Game.sprites['heroesMegaPack']));
        text.setText(Game.strings[Game.language].wonGui.heroPack);
        text.position.y -= 198;
        container.addChild(text);
        container.mousedown = container.touchstart = function () {
            yestochedheroes = true;
        }.bind(this);
        container.mouseup = container.touchend  = container.mouseupoutside = container.touchendoutside = function () {
            if (yestochedheroes) {
                Game.globalHeroIcon = container;
                yestochedheroes = false;
                Game.globallogic.buyHeroPack(function (result) {
                    if (result)
                        Game.lvl.pool.pull(container);
                })
            }
        }.bind(this);
    } else {
        var yestochedscrolls = false;
        container.addChild(Game.lvl.pool.getSprite(Game.sprites['scrollsMegaPack']));
        text.setText(Game.strings[Game.language].wonGui.scrollPack);
        text.position.y -= 198;
        container.addChild(text);
        container.mousedown = container.touchstart = function () {
            yestochedscrolls = true;
        }.bind(this);
        container.mouseup = container.touchend  = container.mouseupoutside = container.touchendoutside = function () {
            if (yestochedscrolls) {
                yestochedscrolls = false;
                Game.globalPackIcon = container;
                Game.globallogic.buyScrollPack(function (result) {
                    if (result)
                        Game.lvl.pool.pull(container);
                })
            }
        }.bind(this);
    }
    return container;
}

function getRandomSkullsOrScrollPrize(withScrollChance) {
    var dice =  Math.random();
    var container = new PIXI.DisplayObjectContainer();
    if (dice > 0.6 || withScrollChance) {
        var sprite = myRandom(0,16);
        switch (sprite) {
            case 0:case 6:case 7:case 14:case 16:case 15:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['goldBag'+'Scroll']));
            Game.globallogic.addItem('goldBag');
            break;
            case 1:case 8:case 12:case 11:case 13:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['bomb'+'Scroll']));
            Game.globallogic.addItem('bomb');
            break;
            case 2:case 9:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['iceStaff'+'Scroll']));
            Game.globallogic.addItem('iceStaff');
            break;
            case 3:case 10:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['meteorite'+'Scroll']));
            Game.globallogic.addItem('meteorite');
            break;
            case 4:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['life'+'Scroll']));
            Game.globallogic.addItem('life');
            break;
            case 5:
            container.addChild(Game.lvl.pool.getSprite(Game.sprites['resurrection'+'Scroll']));
            Game.globallogic.addItem('resurrection');
            break;
        }
    } else {
        Game.fillStyle = "#FFF2B3";
        Game.textAlign = 'left';
        Game.font = Game.getFont(96);
        var skulls = myRandom(30, 61);
        Game.globallogic.addSkulls(skulls);
        var rewardedTextCount = new PIXI.Text('+' + skulls, {
            font: Game.font,
            align: 'left',
            fill: Game.fillStyle,
            wordWrapWidth:200,
            wordWrap:true,
        });
        rewardedTextCount.position = new Vec2(-36,-100);
        container.addChild(rewardedTextCount);
        var rewardedSkullIcon = Game.lvl.pool.getSprite(Game.sprites['skullsShopPriceIcon']);
        rewardedSkullIcon.scale.set(1.5,1.5);
        rewardedSkullIcon.position.set(96,-40);
        container.addChild(rewardedSkullIcon);

    }
    return container;
}

Hero = function(name, purchaseId) {
    this.name = name;
    this.lvl = 1;
    this.experience = 0;
    this.purchaseId = purchaseId;
    this.purchased = !purchaseId;
    this.publicName = this.description =  (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.names[this.name]: 'Fail') : "Fail");
    this.heroIcon = Game.sprites['HeroIcon'+this.name] || Game.sprites['HeroIconwarrior'] ;
    this.portrait = Game.sprites['heroPortrait'+this.name];
    this.iconability1 = Game.sprites['hero'+this.name+'ability1'] || Game.sprites['herowarriorability1'];
    this.iconability2 = Game.sprites['hero'+this.name+'ability2'] || Game.sprites['herowarriorability1'];
    this.description =  (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.description[this.name]: 'Fail') : "Fail");
};

Hero.prototype.reloadIcon = function() {
    this.heroIcon = Game.sprites['HeroIcon'+this.name] || Game.sprites['HeroIconwarrior'] ;
    this.portrait = Game.sprites['heroPortrait'+this.name];
    this.iconability1 = Game.sprites['hero'+this.name+'ability1'] || Game.sprites['herowarriorability1'];
    this.iconability2 = Game.sprites['hero'+this.name+'ability2'] || Game.sprites['herowarriorability1'];
    this.description =  (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.description[this.name]: 'Fail') : "Fail");

};

Hero.prototype.addExperience = function(number) {
    if (this.lvl < 20)
        this.experience+=number;
    else
        return false;

    if (this.experience > this.getEpxNeed()) {
        this.lvl++;
        this.save();
        Game.events.emit('heroLvlUp');
        return true;
    }
    this.save();
    return false;
};

Hero.prototype.getEpxNeed = function() {
    var exp = 0;
    for (var i=2; i<=this.lvl+1; i++) {
        exp+=i*1000;
    }
    if (this.lvl == 20)
        exp-=i*1000;
    return exp;
};


Hero.prototype.isAbilityOpen = function(number) {
    //TODO debug
    return true;
    if (number == 1)
        return true;
    else
        return this.lvl >= (number-1)*10;
};

Hero.prototype.load = function() {
    this.reloadIcon();
    this.experience = parseInt(localStorage.getItem(Game.profile + this.name+'experience')) || 0;
    this.lvl = parseInt(localStorage.getItem(Game.profile + this.name+'lvl')) || 1;
    if (isNaN(this.lvl) || this.lvl > 20 || this.lvl < 1)
        this.lvl = 1;

    if (this.name === 'warrior') {
        this.purchased = true;
    } else if (this.name === 'archer') {
        if (localStorage.getItem('archer') == 'true')
            this.purchased = true;
    } else if (this.name == 'fireMage') {
        if (localStorage.getItem('fireMage') == 'true')
            this.purchased = true;
    } else if (this.name == 'wolfRider'){
        if (localStorage.getItem('wolfRider') == 'true')
            this.purchased = true;
    } else if (this.name == 'ogre'){
        if (localStorage.getItem('ogre') === 'true')
            this.purchased = true;
    } else if (this.name == 'templarAssassin'){
        if (localStorage.getItem('templarAssassin') == 'true')
            this.purchased = true;
    }
};

Hero.prototype.save = function() {
    localStorage.setItem(Game.profile + this.name+'experience', this.experience);
    localStorage.setItem(Game.profile + this.name+'lvl', this.lvl);
};

Hero.prototype.purchase = function () {
    if (this.purchased)
        return;
    this.purchased = true;
    localStorage.setItem(this.name, 'true');
};

Hero.prototype.getLocalizedName = function() {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.names[this.name] : 'Fail') : "Fail")
};

Hero.prototype.getExpPercents = function() {
    return this.experience/(this.lvl * this.lvl * 100);
};

Hero.prototype.getAbilityName = function(number) {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.abilities[this.name][number].name : 'Fail') : "Fail")
};

Hero.prototype.getAbilityDescription = function (number){
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.abilities[this.name][number].description : 'Fail') : "Fail")
};

Hero.prototype.getHeroDescription = function () {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.description[this.name] : 'Fail') : "Fail")
};

Hero.prototype.getHeroName = function () {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].heroes.names[this.name] : 'Fail') : "Fail")
};

Hero.prototype.openLabel = function () {
    if (this.name === 'warrior') {
        return Game.globallogic.lvls[3].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    } else if (this.name === 'archer'){
        return Game.globallogic.lvls[9].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    } else if (this.name == 'fireMage') {
        return Game.globallogic.lvls[19].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    } else if (this.name == 'wolfRider'){
        return Game.globallogic.lvls[3].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    } else if (this.name == 'ogre'){
        return Game.globallogic.lvls[3].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    } else if (this.name == 'templarAssassin'){
        return Game.globallogic.lvls[3].passed > 0 ? false : Game.strings[Game.language].heroes.available[this.name];
    }
    return false;
};

Upgrade = function(key, legalName, cost, dependency, position, chainNext) {
    this.name = key;
    this.legalName = legalName;
    this.cost = cost;
    this.dependency = dependency;
    this.position = position;
    this.purchased = this.cost === 0;
    this.chainNext = chainNext || null;
};

Upgrade.prototype.load = function() {
    var purchased = localStorage.getItem(Game.profile + this.name);
    if (((purchased)&&(purchased!=='false'))||(this.cost === 0)) {
        Game.globallogic.purchase(this);
    }
};

Upgrade.prototype.isAvailable = function() {
    return !this.dependency || this.dependency.purchased;
};

Upgrade.prototype.purchase = function() {
    this.purchased = true;
    localStorage.setItem(Game.profile + this.name, true);
};
Upgrade.prototype.reset = function(){
    this.purchased = this.cost === 0;
    localStorage.setItem(Game.profile + this.name, this.purchased);
};


LvlPrefs = function(key, episode, position){
    this.name = key;
    this.position = position;
    this.episode = episode;
    this.passed = 0;
    this.difficulty = 'easy';
    this.amimationPlay = myPofigRandom(10, 1000);
};

LvlPrefs.prototype.load = function () {
    var passed = parseInt(localStorage.getItem(Game.profile+this.name));
    this.difficulty = localStorage.getItem(Game.profile+this.name+'difficulty') || 'easy';
    if (isNaN(passed)) {
        this.passed = 0;
        localStorage.setItem(Game.profile + this.name, 0);
    } else {
        this.passed = parseInt(passed);
        Game.globallogic.allStars += this.passed;
    }
};

LvlPrefs.prototype.nameToPrint = function () {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].lvls[this.name].name : 'Fail') : "Fail");
};

LvlPrefs.prototype.descriptionToPrint = function () {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].lvls[this.name].description : 'Fail') : "Fail");
};

function difficultyToNumber (dif) {
    switch (dif) {
        case 'easy':
            return 0;
            break;
        case 'normal':
            return 1;
            break;
        case 'hard':
            return 2;
            break;
        default:
            return 0;
            break;
    }
}

LvlPrefs.prototype.setPassed = function (passed) {
    if (passed > this.passed) {
        localStorage.setItem(Game.profile + this.name, passed);
        Game.globallogic.availableStars += passed - this.passed;
        Game.globallogic.allStars += passed - this.passed;
        this.passed = passed;
    }

    if (difficultyToNumber(Game.globallogic.difficulty) > difficultyToNumber(this.difficulty)) {
        this.difficulty = Game.globallogic.difficulty;
        localStorage.setItem(Game.profile+this.name+'difficulty', this.difficulty);
    }
};

LvlPrefs.prototype.isOpen = function () {
    //return true;

    if (this.name == 0){
        return true;
    } else {
        var prevLvl = parseInt(this.name)- 1;
        if (this.episode == 'forest' && parseInt(this.name) == 10)
            prevLvl = 7;
        else if (this.episode == 'graveyard' && parseInt(this.name) == 20)
            prevLvl = 18;
        var dependency = Game.globallogic.getLvlWithName(prevLvl);
        return !!((!dependency) || (dependency.passed > 0));
    }
};

LvlPrefs.prototype.drawFlag = function () {
    var animation = Game.animations['BannerAlly'];
    this.amimationPlay += Game.cycleTickTime;

    switch (this.episode) {
        case 'desert':
            animation = Game.animations['BannerOrcs'];
            break;
        case 'forest':
            animation = Game.animations['BannerElfes'];
            break;
        case 'graveyard':
            animation = Game.animations['BannerNecro'];
            break;
    }

    if (this.isOpen()) {
        Game.ctx.translate(this.position.x, this.position.y);
        Game.sprites['FlagPoint'].draw();
        animation.drawFrame((this.amimationPlay % animation.playTime).div(animation.playTime/animation.count));
        Game.ctx.translate(-this.position.x, -this.position.y);
    }
};

LvlPrefs.prototype.drawPath = function () {
    var pathName = 'Path' + this.name + '.png';
    var sprite = Game.sprites[pathName];
    if ((sprite)&&(this.isOpen())) {
        Game.ctx.translate(this.position.x*2, this.position.y*2);
        sprite.draw();
        Game.ctx.translate(-this.position.x*2, -this.position.y*2);
    }
};

LvlPrefs.prototype.checkTouch = function (touchPosition) {
    if (this.isOpen()){
        return (this.position.clone().sub(touchPosition).getLength() < 100);
    } else
        return false;
};

LvlPrefs.prototype.getLocalizedName = function (key) {
    return (Game.strings ? (Game.strings[Game.language] ? Game.strings[Game.language].lvls[key].name : 'Fail') : "Fail")
};

function xhrPost(url, data, callback){
    var xhr = new XMLHttpRequest();
    data = JSON.stringify(data);
    xhr.open("post", 'http://crusade.mnillstone.com:82/' + url, true);
    xhr.setRequestHeader("Content-Type", "application/json");
    xhr.setRequestHeader("Content-length", ''+data.length);

    xhr.onreadystatechange = function() {//Call a function when the state changes.
        if(xhr.readyState == 4) {
            if (xhr.status == 200){
                callback && callback(null, xhr.responseText);
            } else {
                callback && callback(xhr.statusText, xhr.responseText);
            }
        }
    };
    xhr.send(data);
}