﻿function Gui(params) {
    this.base = params;
    this.sprite = params.sprite;
    this.x = params.x;
    this.y = params.y;
    this.animation = null;
    this.animationIterator = 0;
    this.animationDelay = 0;
    this.animationCallback = null;
    this.animationRevers = false;
    this.width = params.width;
    this.height = params.height;
    this.onTouch = params.onTouch || function(){};
    this.draw = params.draw;
    this.hidden = false;

    if (params.noLock)
        this.lock = false;
    else
        this.lock = true;

    if (params.init)
        params.init.call(this, params);

    if (params.update)
        this.onUpdate = params.update;

}

Gui.prototype.update  = function () {

    if ((this.animation != null)&&(this.animationIterator > 0)){
        if (this.animationDelay > 0){
            this.animationDelay--;
            return;
        }
        this.x += (this.animation.toX - this.animation.startX) * (this.animationRevers ? -1 : 1) / this.animation.iterations;
        this.y += (this.animation.toY - this.animation.startY) * (this.animationRevers ? -1 : 1) / this.animation.iterations;
        if (this.guiContainer)
            this.guiContainer.position.set(this.x + this.width/2, this.y + this.height/2);


        if ((--this.animationIterator == 0)&&(this.animationCallback != null)) {
            this.animationCallback();
        }

        if (this.guiContainer)
            this.guiContainer.position.set(this.x+this.width/2, this.y+this.height/2);
    }
    if (this.onUpdate)
        this.onUpdate();
};

Gui.prototype.checkOpen = function (){
    return true;
};

Gui.prototype.draw = function (){
    if (this.hidden)
        return;
    //Gamectx..drawImage(this.sprite.getImage(), 0, 0, this.sprite.getImage().width,  this.sprite.getImage().height, this.xOnScreen , this.yOnScreen, this.width, this.height);
};

Gui.prototype.checkTouch  = function(touch){
    if (!this.hidden && this.onTouch &&(touch.startPosition.x > this.x)&& (touch.startPosition.x<this.x+this.width) && (touch.startPosition.y > this.y )&& (touch.startPosition.y<this.y+this.height))
    {
        this.onTouch(touch);
        return true;
    }
    return false;
};

Gui.prototype.checkMenuTouch  = function(touch){
    if ((!this.hidden)&&(touch.startPosition.x > this.x/Game.scale)&& (touch.startPosition.x<(this.x+this.width)/Game.scale) && (touch.startPosition.y > this.y/Game.scale)&& (touch.startPosition.y<(this.y+this.height)/Game.scale))
    {
        this.onTouch(touch);
        return true;
    }
    return false;
};

Gui.prototype.isLocked = function(x, y){
    if (x instanceof Vec2){
        y = x.y;
        x = x.x;
    }
    if (this.lock){
        if ((x > this.x)&& (x<this.x+this.width) && (y > this.y )&& (y<this.y+this.height))
            return true;
    }
    return false;
};

Gui.prototype.checkLastTouch = function (last) {
    return ((last.x > this.x)&& (last.x<this.x+this.width) && (last.y > this.y )&& (last.y<this.y+this.height))
};

Gui.prototype.startAnimation = function(animation, callback, reverse, delay){

    if (!animation)
        return;
    if (!reverse){
        this.x = animation.startX;
        this.y = animation.startY;
    } else {
        this.x = animation.toX;
        this.y = animation.toY;
    }

    this.animationRevers = reverse || false;
    this.animationDelay = delay || 0;
    this.animation = animation;
    this.animationIterator = animation.iterations;
    this.animationCallback = callback || null;
    if (this.guiContainer)
        this.guiContainer.position.set(this.x + this.width/2, this.y + this.height/2);
};


Gui.prototype.show = function(){

};


Gui.prototype.hide = function(){

};
