﻿function Vec2(x, y) {
	this.x = x;
	this.y = y;
}

// Get rotate from move vector!
Vec2.prototype.getRotate = function (){
	var rotate = Math.acos((0*this.x +1*this.y)/(Math.sqrt((Math.pow(0,2)+Math.pow(1,2))*(Math.pow(this.x,2)+Math.pow(this.y,2)))));
	if (this.x < 0) 
		rotate = Math.PI*2-rotate;
	if (isNaN(rotate))
		return 0;
	return rotate;
};
   

Vec2.prototype.getRotated = function (angle){
    return this.clone().rotate(-angle);
};
   

Vec2.prototype.rotate = function (angle){
	var cachePoint = new Vec2(this.x, this.y);
	this.x  = cachePoint.x * Math.cos(angle) - cachePoint.y * Math.sin(angle);
	this.y  = cachePoint.x * Math.sin(angle) + cachePoint.y * Math.cos(angle);
	return this;
};


Vec2.prototype.multiply = function (factor){
	this.x*=factor; 
	this.y*=factor;
	return this;
};


Vec2.prototype.multiplyed = function (factor){
	return new Vec2(this.x*factor, this.y*factor);
};

	
Vec2.prototype.add = function (vecToAdd){
	if (!(vecToAdd instanceof Vec2)&&(arguments.length == 2))
		vecToAdd = new Vec2(arguments[0], arguments[1]);
	this.x+=vecToAdd.x;
	this.y+=vecToAdd.y;
	return this;
};
	

Vec2.prototype.sub = function (vecToSub){
	this.x-=vecToSub.x;
	this.y-=vecToSub.y;
	return this;
};
	

Vec2.prototype.clone = function () {
	return new Vec2(this.x, this.y);
};

Vec2.prototype.set = function (x, y) {
	if (x instanceof Vec2){
		this.x = x.x;
		this.y = x.y;
	} else {
		this.x = x;
		this.y = y;
	}
};


Vec2.prototype.normalize = function (){
	var length = this.getLength();
	if (length){
	this.multiply(1/this.getLength());
	}
	return this;	
};
	

Vec2.prototype.getLength = function (){
	return Math.sqrt(Math.pow(this.x,2)+Math.pow(this.y,2));
};

Vec2.prototype.getDirection = function (){
	if (this.x > 0)
			return 1;
		else
			return -1;
};


Vec2.prototype.getPositionOnScreen = function (pCircleRadius){
	if (!pCircleRadius) 
		pCircleRadius =0;
	var onScreen = new Vec2(this.x-(Game.lvl.player.pObj.position.x-Game.width/2), this.y-(Game.lvl.player.pObj.position.y-Game.height/2));
    if ((onScreen.x<-pCircleRadius)||(onScreen.y<-pCircleRadius)||(onScreen.x>Game.width+pCircleRadius)||(onScreen.y>Game.height+pCircleRadius))
		return null;
	else
		return onScreen;
};



function pObject(x, y, width, height, direction, moveDirection, isGravity, parent) {
	this.parent = parent;
	this.width = width;
	this.height = height;
	this.pCircleRadius = Math.max(width, height)/2;
	this.position = new Vec2(x,y);
 	this.direction = direction;
	this.mass = parent.mass;
}
   

pObject.prototype.move = function(moveDirection){
 		this.position.add(moveDirection);
};
   

pObject.prototype.checkCollision = function(mask){
	//if ((this.position.x<0)||(this.position.y<0)||(this.position.x>Game.mapWidth)||(this.position.y>Game.mapHeight))
		//return "out";
		
	for (var i=0; i< mask.length; i++)
		{
			for (var i1=0; i1< mask[i].length; i1++)
			{
				if ((Math.abs(mask[i][i1].pObj.position.x - this.position.x)< Math.max( this.pCircleRadius , mask[i][i1].pObj.pCircleRadius))&&(Math.abs(mask[i][i1].pObj.position.y - this.position.y)< Math.max( this.pCircleRadius ,mask[i][i1].pObj.pCircleRadius))) 	
				{
				if(mask[i][i1] !=this.parent)
					return mask[i][i1];
				}
			}
		}
	return false;
};


pObject.prototype.rotateTo = function(angle, rotateSpeed){
	var diff = this.direction.getRotate();
	var rotate = Math.abs(diff - angle) > rotateSpeed*Game.lvl.interval ? rotateSpeed*Game.lvl.interval : Math.abs(diff - angle);
    if (Math.abs(diff - angle)+rotate > Math.PI*2) rotate = Math.PI*2 - Math.abs(diff - angle);
		if (((angle > diff)&&(Math.abs(angle-diff)<Math.PI))||(angle < diff)&&(Math.abs(angle-diff)>Math.PI)){
			this.direction.rotate(-rotate);
			return -rotate;
			} else {
			this.direction.rotate(rotate);
			return rotate;
			}
};
	

pObject.prototype.getPositionOnScreen = function(){	
	return this.position.getPositionOnScreen(this.pCircleRadius);
};
	

pObject.prototype.collision = function(collisionTarget){
	var strength = this.moveDirection.clone().sub(collisionTarget.pObj.moveDirection).getLength()*(1-this.mass/(this.mass+collisionTarget.pObj.mass));
	this.parent.applyDamage(strength/10);
	this.applyPhysic(new Vec2(this.position.x-collisionTarget.pObj.position.x, this.position.y-collisionTarget.pObj.position.y).normalize().multiply(
	1+strength))
};