﻿function Sprite(data) {
    this.x = data.frame.x;
    this.y = data.frame.y;
    this.width = data.frame.width;
    this.height = data.frame.height;
    this.center = data.movedCenter;
    this.drawX = - data.frame.width/2;
    this.drawY = - data.frame.height/2;
    if (!Game.atlases.hasOwnProperty(data.atlas))
        Game.atlases[data.atlas]=new Atlas('atlases/'+data.atlas, data.tag);
    this.atlas = Game.atlases[data.atlas];
}

Sprite.prototype.draw = function ()
{
    Game.ctx.translate(this.center[0], this.center[1]);
    Game.ctx.drawImage(this.atlas.image, this.x, this.y, this.width, this.height, this.drawX, this.drawY, this.width, this.height);
    Game.ctx.translate(-this.center[0], -this.center[1]);
};

Sprite.prototype.drawMultiplied = function (counter)
{
    Game.ctx.translate(this.center[0], this.center[1]);
    Game.ctx.drawImage(this.atlas.image, this.x, this.y, this.width, this.height, this.drawX*counter, this.drawY*counter, this.width*counter, this.height*counter);
    Game.ctx.translate(-this.center[0], -this.center[1]);
};


function Atlas(fileName, tag) {
    this.fileName = fileName;
    this.tag = tag;
    this.loaded = false;
    Game.atlasList.push(this);
}

Atlas.prototype.load = function(onLoad) {
    this.loader = new PIXI.loaders.Loader();
    this.loader.add(this.fileName,this.fileName);
    this.loader.on('complete', function(){
        onLoad();
    }.bind(this));
    this.loader.load();
    this.loaded = true;
};

Atlas.prototype.unload = function () {
    this.image = null;
    if (this.loader) {
        this.loader.resources[this.fileName + '_image'].data.dispose && this.loader.resources[this.fileName + '_image'].data.dispose();
        this.loader.resources[this.fileName + '_image'].texture.baseTexture.destroy();
        this.loader.resources[this.fileName + '_image'].texture.destroy(true);
        this.loader = null;
    }
    this.loaded = false;
};


Atlas.loadAtlasesForEpisode = function(episode, callback, onProgress) {
    var loadedAtlases = [];
    var key, lastProgress = Date.now();

    Game.TextTexturesCache.forEach(function(texture){
        if (Cocoon.nativeAvailable && texture._texture && texture._texture.baseTexture && texture._texture.baseTexture.source && texture._texture.baseTexture.source.dispose) {
            texture._texture.baseTexture.source.dispose();
        }
        texture.destroy(true);
        texture.source = null;
    });
    Game.TextTexturesCache = [];
    Game.lvl.achievementsText = null;
    Game.globalHeroIcon = null;
    Game.globalPackIcon = null;
    Game.purchaseEpisodeCallback = null;
    Game.onresize = [];

    Game.previewBackground && Game.previewBackground.unload() && (Game.previewBackground = null);

    Game.currentPlayedMusic && Game.currentPlayedMusic.stop();
    Game.currentPlayedMusic = null;
    Game.audioList.forEach(function(audio){
        audio.stop && audio.stop();
        audio.dispose && audio.dispose();
    });
    Game.audioList = [];

    if ((Game.release && !Game.pc) || navigator.isCocoonJS && Cocoon.nativeAvailable)
        for (key in Game.atlases)
            if (Game.atlases[key].tag.indexOf('default') == -1 && (Game.atlases[key].tag.indexOf(episode) ==  -1) && (Game.atlases[key].tag.indexOf('loading') === -1 || episode != 'gui') &&( Game.atlases[key].tag != 'particles'))
                Game.atlases[key].unload();

    var atlasToLoadCounter = 0, allAtlases = 0;

    var onAtlasLoad = function() {
        if (Date.now() - lastProgress < 500) {
            setTimeout(function() {
                if (onProgress)
                    onProgress(1-atlasToLoadCounter/allAtlases);
                if (--atlasToLoadCounter === 0) {
                    onProgress(1);
                    setTimeout(callback, 100);
                }
            }, (lastProgress + 500) - Date.now());
            lastProgress += 500;
        } else {
            lastProgress = Date.now();
            if (onProgress)
                onProgress(1-atlasToLoadCounter/allAtlases);
            if (--atlasToLoadCounter === 0)
                setTimeout(callback, 15000);
        }
    };


    var load = [];
    for (key in Game.atlases) {
        if (Game.atlases[key].loaded != true && ((!Cocoon.nativeAvailable && Game.pc) || Game.atlases[key].tag.indexOf(Game.menu.selectedLvl == "0" ? 'warrior' : Game.globallogic.selectedHero.name) != -1 ||  Game.atlases[key].tag.indexOf('loading')  != -1 || Game.atlases[key].tag.indexOf(episode) != -1 || Game.atlases[key].tag.indexOf('default') != -1 || Game.atlases[key].tag == 'particles')) {
            atlasToLoadCounter++;allAtlases++;
            load.push(Game.atlases[key]);
        }
    }

    if (Game.pc) {
        Game.backgroundsList.forEach(function (back) {
            if (!back.loader) {
                load.push(back);
                atlasToLoadCounter++;
                allAtlases++;
            }
        });
        Game.backgroundsPreviewList.forEach(function (back) {
            if (!back.loader) {
                load.push(back);
                atlasToLoadCounter++;
                allAtlases++;
            }
        });
    }

    if (load.length > 0) {
        if (Cocoon.nativeAvailable) {
            loadSeries(load, callback, onProgress);
        } else {
            load.forEach(function(atlas) {
                atlas.load(onAtlasLoad);
            })
        }
    }

    if (atlasToLoadCounter == 0) {
        virtualProgress(0, onProgress, callback);
    }
};

function loadSeries(arr, callback, onProgress, it) {
    var start = Date.now();
    if (!it)
        it = 0;

    arr[it].load(function(){
        if (Date.now() - start < 10000) {
            setTimeout(function() {
                if (onProgress)
                    onProgress(it/arr.length);
                if (++it >= arr.length) {
                    onProgress(1);
                    setTimeout(callback, Game.first ? 100 : 8000);
                } else
                    loadSeries(arr, callback, onProgress, it);

            }, (start + 100) - Date.now());
        } else {
            if (onProgress)
                onProgress(it/arr.length);
            if (++it >= arr.length)
                setTimeout(callback, Game.first ? 100 : 8000);
            else
                loadSeries(arr, callback, onProgress, it);
        }
    })
}

function virtualProgress(count, onprogress, callback) {
    if (count >= 1) {
        onprogress(1);
        setTimeout(callback, 10000);
    } else {
        onprogress(count);
        setTimeout(function(){virtualProgress(count+0.005, onprogress, callback)}, 10);
    }
}

Atlas.loadLoader = function(callback){
    var key;

    var atlasToLoadCounter = 0;
    var onAtlasLoad = function() {
        if (--atlasToLoadCounter === 0)
            callback();
    };

    for (key in Game.atlases) {
        if (Game.atlases[key].loaded != true && (Game.atlases[key].tag.indexOf('loading') != -1 || Game.atlases[key].tag == 'particles')) {
            atlasToLoadCounter++;
            Game.atlases[key].load(onAtlasLoad);
        }
    }

    if (atlasToLoadCounter == 0){
        callback();
    }
};

function Background(name) {
    this.textures = [];
    this.loader = null;
    this.name = name;
    this.src = 'backgrounds/'+name;
    this.container = new PIXI.Container();
}

Background.prototype.load = function(callback) {
    if (!this.loader) {
        this.loader = new PIXI.loaders.Loader();
        this.loader.add(this.src, this.src);
        this.loader.on('complete', function (res) {
            this.textures.push(this.loader.resources[this.src].texture);
            var sp1 = new PIXI.Sprite(this.loader.resources[this.src].texture);
            this.container.addChild(sp1);
            callback();
        }.bind(this));
        this.loader.load();
        this.loaded = true;
    } else {
        setTimeout(function () {
            callback();
        },1);
    }
};

Background.prototype.unload = function(){
    if (this.loader) {
        this.textures.forEach(function(texture) {
            texture.baseTexture &&  texture.baseTexture.dispose && texture.baseTexture.dispose();
            texture.destroy(true);
        });
        this.loader = null;
        Game.cachedBackground = null;
    }
};

function BackgroundPreview(name){
    this.textures = [];
    this.loader = null;
    this.src = 'backgroundsPreview/'+name + '.jpg';
    this.name = name;
    this.container = new PIXI.Container();
}

BackgroundPreview.prototype.load = function(callback) {
    if (!this.loader) {
        this.loader = new PIXI.loaders.Loader();
        this.loader.add(this.src, this.src);
        this.loader.on('complete', function(res) {
            this.textures.push(this.loader.resources[this.src].texture);
            var sp1 = new PIXI.Sprite(this.loader.resources[this.src].texture);
            sp1.anchor.set(0.5,0.5);
            this.container.addChild(sp1);
            callback && callback();
        }.bind(this));
        this.loader.load();
        this.loaded = true;
    } else {
        setTimeout(function(){
            callback && callback();
        },1);
    }
};

BackgroundPreview.prototype.unload = function(){
    if (this.loader && !Game.pc) {
        this.textures.forEach(function(texture) {
            texture.baseTexture &&  texture.baseTexture.dispose && texture.baseTexture.dispose();
            texture.destroy(true);
        });
        this.loader = null;
    }
};


function EpisodePreview(name){
    this.textures = [];
    this.loader = null;
    this.src = 'episodesPreview/'+name;
    this.container = new PIXI.Container();
}

EpisodePreview.prototype.load = function(callback) {
    if (!this.loader) {
        this.loader = new PIXI.loaders.Loader();
        this.loader.add(this.src +'_1.jpg', this.src +'_1.jpg');
        this.loader.add(this.src +'_2.jpg', this.src +'_2.jpg');
        this.loader.on('complete', function(res) {
            this.textures.push(this.loader.resources[this.src +'_1.jpg'].texture);
            this.textures.push(this.loader.resources[this.src +'_2.jpg'].texture);
            var sp1 = new PIXI.Sprite(this.loader.resources[this.src + '_1.jpg'].texture);
            sp1.position.set(0, 0);

            var sp2 = new PIXI.Sprite(this.loader.resources[this.src +'_2.jpg'].texture);
            sp2.position.set(0, 511);

            this.container.addChild(sp1);
            this.container.addChild(sp2);
            callback && callback();
        }.bind(this));
        this.loader.load();
        this.loaded = true;
    } else {
        setTimeout(function(){
            callback && callback();
        },1);
    }
};

EpisodePreview.prototype.unload = function(){
    if (this.loader) {
        this.textures.forEach(function(texture) {
            texture.baseTexture &&  texture.baseTexture.dispose && texture.baseTexture.dispose();
            texture.destroy(true);
        });
        this.loader = null;
    }
};