﻿function SpritePool() {
    this.animations = {};
    this.sprites = {};
};

SpritePool.prototype.pull = function(obj){
    if (obj instanceof PIXI.extras.MovieClip){
        this.pullAnimation(obj);
    } else if (obj instanceof PIXI.Text){
        if (obj.parent)
            obj.parent.removeChild(obj);
    } else if (obj instanceof PIXI.Sprite){
        obj.scale = new Vec2(1,1);
        this.pullSprite(obj);
    } else {
        if (obj && obj.parent)
            obj.parent.removeChild(obj);
    }
};

SpritePool.prototype.getSprite = function(name){
    var sprite;
    if (!this.sprites[name.name])
        this.sprites[name.name] = [];

    if (this.sprites[name.name].length > 0) {
        sprite = this.sprites[name.name].pop();
    } else {
        sprite = PIXI.Sprite.fromFrame(name.name);
        sprite.anchor = new PIXI.Point((sprite.texture.width + sprite.texture.basePivot.x)/sprite.texture.width - 0.5, (sprite.texture.height + sprite.texture.basePivot.y)/sprite.texture.height -0.5);
    }
    sprite.setDirection(1);
    sprite.name = name.name;
    return sprite;
};

//new PIXI.Point(rect.w/2-frameData[i].originalCenter[0], rect.h/2-frameData[i].originalCenter[1])
SpritePool.prototype.pullSprite = function(sprite){
    if (!this.sprites[sprite.name])
        this.sprites[sprite.name] = [];

    if ( this.sprites[sprite.name].indexOf(sprite) === -1)
        this.sprites[sprite.name].push(sprite);

    if (sprite.parent)
        sprite.parent.removeChild(sprite);
};

SpritePool.prototype.switchSprite = function(name, sprite){
    if (sprite.name == name.name)
        return sprite;

    var newSprite = this.getSprite(name);
    newSprite.position = sprite.position;
    if (sprite.parent) {
        sprite.parent.addChildAt(newSprite, sprite.parent.getChildIndex(sprite));
    }
    this.pullSprite(sprite);
    return newSprite;
};


SpritePool.prototype.getAnimation = function(name){
    var anim;

    if (!this.animations[name])
        this.animations[name] = [];

    if (this.animations[name].length > 0) {
        anim = this.animations[name].pop();
        anim._currentTime = 0;
        anim.currentFrame = 0;
        anim._texture = anim._textures[0];
    } else {
        anim = PIXI.extras.MovieClip.fromFrames(Game.animations[name].sprites);
        //anim.anchor = new PIXI.Point((anim.texture.width + anim.texture.basePivot.x)/anim.texture.width -0.5, (anim.texture.height + anim.texture.basePivot.y)/anim.texture.height -0.5);

        anim.animationSpeed = Game.animations[name].playTime;
        anim.name = name;
        anim.pivot = anim.texture.framePivot;
    }
    anim.play();
    anim.setDirection(1);
    return anim;
};

SpritePool.prototype.pullAnimation = function(animation){
    if (!this.animations[animation.name])
        this.animations[animation.name] = [];

    animation.stop();
    animation.onComplete = null;

    if (this.animations[animation.name].indexOf(animation) === -1)
        this.animations[animation.name].push(animation);

    if (animation.parent)
        animation.parent.removeChild(animation);
};

SpritePool.prototype.switchAnimation = function(name, anim){
    if (anim.name == name)
        return anim;

    var newAnim = this.getAnimation(name);
    newAnim.position = anim.position;
    newAnim.onComplete = anim.onComplete;
    newAnim._currentTime = 0;
    if (anim.direction)
        newAnim.setDirection(anim.direction);
    if (anim.parent)
        anim.parent.addChildAt(newAnim, anim.parent.children.indexOf(anim));

    this.pullAnimation(anim);
    return newAnim;
};