﻿function Tile(params, position, positionInArray) {
    this.position = position;
    this.positionInArray = positionInArray;
    this.spawnTags = [];
    this.parentTile= null;
    this.isGlobalyWalkable = true;
    this.isBusy = false;
    this.isWalkable = true;
    this.flyAble = false;
    this.anchor = false;
    this.isCristal = false;
    this.envObjects = [];
    this.objects = [];
    this.wObjects = [];
    this.eObjects = [];
    this.onBuild = [];
    this.specialColor = [];
    this.buildingsAround = 0;
    this.buildingSprite = null;
}

Tile.prototype.update  = function () {
    //Nothing there
};

Tile.prototype.setFullUnwalkAble = function (){
    this.isGlobalyWalkable = false;
    this.isWalkable = false;
};

Tile.prototype.setUnwalkAble = function (){
    this.isWalkable = false;
};


Tile.prototype.setAnchor = function() {
    this.anchor = true;
};

Tile.prototype.setCristal = function() {
    new Flag(this);
    this.cristal = true;
    this.setFullUnwalkAble();
};

Tile.prototype.setWalkable = function(value){
    this.isWalkable = value;

    for (var obj in this.onBuild)
        this.onBuild[obj]()
};

Tile.prototype.setFlyAble = function(value){
    this.flyAble = value;
};

Tile.prototype.setATile = function(){
    this.specialColor.push('a');
};

Tile.prototype.setBTile = function(){
    this.specialColor.push('b');
};

Tile.prototype.setCTile = function(){
    this.specialColor.push('c');
};

Tile.prototype.clone  = function () {
    var tile;

    tile = new Tile(null, this.position, this.positionInArray);
    tile.parentTile = this;
    tile.isWalkable = this.isWalkable;
    return tile;
};

Tile.prototype.debugdraw = function (){
    if (this.canBePlaced()){
        Game.ctx.translate(this.position.x , this.position.y);
        Game.sprites.ground.draw();
        Game.ctx.translate(-this.position.x , -this.position.y);
    }
};

Tile.prototype.debugdrawcristal = function (){
    if (this.canBePlacedCristal()){
        Game.ctx.translate(this.position.x , this.position.y);
        Game.sprites.ground.draw();
        Game.ctx.translate(-this.position.x , -this.position.y);
    }
};


Tile.prototype.debugdraw2 = function (){
    Game.ctx.fillStyle = "black";
    Game.ctx.fillRect(this.position.x - 30, this.position.y - 30, 60, 60);
};


Tile.prototype.canBePlaced = function(){
    if (this.objects.length)
        return false;

    return this.anchor;
};

Tile.prototype.canBePlacedCristal = function(){
    return false;
};


Tile.prototype.canBeWalk = function(){
    if (!this.isGlobalyWalkable)
        return false;

    if (this.isBusy)
        return false;

    if (this.objects.length)
        return false;

    return true;
};


Tile.prototype.predraw = function (){
    if (this.envObjects.length){
        for (var obj in this.envObjects)
            this.envObjects[obj].draw(this);
    }
};

function Flag (tile) {
    this.empty = true;
    this.oldSelected = false;
    this.position = tile.position;
    this.tile = tile;
    this.units = [];
    this.disabled = 0;
    Game.lvl.objEffect.push(this);
    Game.lvl.flags.push(this);
    this.sprite = Game.lvl.pool.getAnimation('BBunnerN');
    this.sprite.position = tile.position.clone();
    Game.lvl.gameContainer.addChild(this.sprite);
}

Flag.prototype.update  = function () {
    var oldFrame;
    this.empty = this.tile.objects.length === 0;
    this.sprite.alpha = this.tile.objects.length > 0 ? 0 : 1;
    if (this.disabled > 0) {
        this.disabled -= Game.cycleTickTime*Game.speed;
    }
    if (Game.lvl.objGui[0].selectedEffect && Game.lvl.objGui[0].selectedEffect.target === this) {
        if (this.oldSelected != true) {
            this.oldSelected = true;
            oldFrame = this.sprite.currentFrame;
            this.sprite = Game.lvl.pool.switchAnimation('BBunnerS', this.sprite);
            this.sprite.currentFrame = oldFrame;

        }
    } else {
        if (this.oldSelected) {
            this.oldSelected = false;
            oldFrame = this.sprite.currentFrame;
            this.sprite = Game.lvl.pool.switchAnimation('BBunnerN', this.sprite);
            this.sprite.currentFrame = oldFrame;
        }
    }
};

Flag.prototype.remove  = function () {
    Game.lvl.pool.pull(this.sprite);

    this.units.forEach(function(unit) {
        unit.del();
    });

    if (Game.lvl.objEffect.indexOf(this) != -1)
        Game.lvl.objEffect.splice(Game.lvl.objEffect.indexOf(this), 1);

    var building = Game.lvl.objBuildings.filter(function(building){return building.position.x == 1056 && building.position.y == 864}).pop();
    building && building.del();
};
